SpojStav subclass: #Relay
	instanceVariableNames: 'leftLow leftHigh rightLow rightHigh '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Relay comment:
' Trida ''Relay'' vytvari funkcni bloky ''Rele'' (stavove bloky).
Instancni promenne prebira po svem otci.

Promenne instanci:
leftLow <Double>  - parametr funkcniho bloku
leftHigh <Double>  - parametr funkcniho bloku
rightLow <Double>  - parametr funkcniho bloku
rightHigh <Double>  - parametr funkcniho bloku'!


!Relay methodsFor: 'accessing'!

lHigh
	"Vraci 'leftHigh'."

	^leftHigh!

lLow
	"Vraci 'leftLow'."

	^leftLow!

rHigh
	"Vraci 'rightHigh'."

	^rightHigh!

rLow
	"Vraci 'rightLow'."

	^rightLow! !

!Relay methodsFor: 'initialization'!

initialize: newName low: newLow high: newHigh lLow: newlLow lHigh: newlHigh rLow: newrLow rHigh: newrHigh initVal: newInitVal 
	"Inicializuje atributy 'Relay' - rele."

	newrLow >= newrHigh ifTrue: [MySimulation error: ' Parametr rLow>=rHigh u tridy Relay.'].
	rightLow := newrLow asDouble.
	rightHigh := newrHigh asDouble.
	newlLow >= newlHigh ifTrue: [MySimulation error: ' Parametr lLow>=lHigh u tridy Relay.'].
	leftLow := newlLow asDouble.
	leftHigh := newlHigh asDouble.
	newLow >= newHigh ifTrue: [MySimulation error: ' Parametr low>=high u tridy Relay.'].
	super
		initialize: newName
		low: newLow
		high: newHigh.
	newInitVal > high
		ifTrue: [MySimulation error: ' Spatna inicializace rele.']
		ifFalse: [newInitVal < low ifTrue: [MySimulation error: ' Spatna inicializace rele.']].
	super initVal: newInitVal!

initVal: newInitVal 
	"Nastavi inicializacni hodnotu bloku 'Relay'."

	newInitVal > high
		ifTrue: [MySimulation error: ' Spatna inicializace rele.']
		ifFalse: [newInitVal < low ifTrue: [MySimulation error: ' Spatna inicializace rele.']].
	super initVal: newInitVal! !

!Relay methodsFor: 'task language'!

eval
	"Nastavuje hodnotu vystupu z rele."

	| y |
	st := input value asDouble.
	st < leftLow
		ifTrue: [y := low]
		ifFalse: [st < leftHigh
				ifTrue: [y := stl]
				ifFalse: [st < rightLow
						ifTrue: [y := 0]
						ifFalse: [st < rightHigh
								ifTrue: [y := stl]
								ifFalse: [y := high]]]].
	y ~= stl ifTrue: [MySimulation active contractStep].
	st := y asDouble! !

!Relay methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor rele."

	SimInspectorView open: ((RelayInspector inspect: self)
			initialize: simModel)
		label: 'Rele `' , name , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Relay class
	instanceVariableNames: ''!


!Relay class methodsFor: 'instance creation'!

new: newName low: newLow high: newHigh lLow: newlLow lHigh: newlHigh rLow: newrLow rHigh: newrHigh 
	"Vytvari a inicializuje blok 'Relay' - rele."

	^super new
		initialize: newName
		low: newLow
		high: newHigh
		lLow: newlLow
		lHigh: newlHigh
		rLow: newrLow
		rHigh: newrHigh
		initVal: newLow!

new: newName low: newLow high: newHigh lLow: newlLow lHigh: newlHigh rLow: newrLow rHigh: newrHigh initVal: newInitVal 
	"Vytvari a inicializuje blok 'Relay' - rele."

	^super new
		initialize: newName
		low: newLow
		high: newHigh
		lLow: newlLow
		lHigh: newlHigh
		rLow: newrLow
		rHigh: newrHigh
		initVal: newInitVal! !

ZaklSpoj subclass: #Condition
	instanceVariableNames: 'cc ccl condition eval '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Condition comment:
' Trida ''Condition'' je bazovou tridou pro vsechny podminky pouzivane
ve spojite simulaci. Zajistuje zarazeni podminky do pole podminek a osetreni
podminky v prubehu simulace.

Promenne instanci:
cc <Double>       - aktualni hodnota podminky
ccl <Double>      - minula hodnota podminky
condition <Block> - akce vyhodnocovana pri splneni podminky
eval <Boolean>    - priznak vyhodnocovani teto podminky'!


!Condition methodsFor: 'accessing'!

condAction
	"Vraci blok, ktery se provede pri splneni podminky."

	^condition! !

!Condition methodsFor: 'initialization'!

initialize: newName 
	"Inicializuje podminku a prirazuje ji jmeno."

	super initialize: newName.
	cc := 0.
	ccl := 0!

initVal
	"Inicializuje podminku."

	cc := 0.
	ccl := 0! !

!Condition methodsFor: 'task language'!

action
	"Provede udalost, ktera je definovana pro tuto podminku."

	condition value!

change
	"Vraci true nebo false podle toho, zda je 'cc' = 'ccl'. Zmena podminky."

	^cc ~= ccl!

setAction: aBlock 
	"Definuje akci, ktera se provede pri platne podmince."

	condition := aBlock!

setValue
	"Nastavuje aktualni hodnotu podminky."

	ccl := cc asDouble!

test
	"Test zmeny podminky."

	| x |
	eval ifTrue: [MySimulation error: 'V simulacnim modelu je algebraicka smycka.'].
	eval := true.
	x := input value asDouble.
	eval := false.
	MySimulation active dyn
		ifTrue: 
			[x ~= 0
				ifTrue: [cc := 1]
				ifFalse: [cc := 0].
			self change
				ifTrue: 
					[MySimulation active con: true.
					MySimulation active contractStep].
			^false].
	^self change!

value
	"Vraci hodnotu podminky."

	^cc asDouble! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Condition class
	instanceVariableNames: ''!


!Condition class methodsFor: 'instance creation'!

new: newName 
	"Vytvari novou instanci tridy 'Condition' a inicializuje ji."

	^MySimulation active condNew: (super new initialize: newName)! !

Object subclass: #Graph
	instanceVariableNames: 'fName step output input file graph graphView time '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Graph comment:
' Trida ''Graph'' vytvari moznost zapisovat vysledky spojite simulace
do grafu. Vysledky se mohou zapisovat do souboru nebo tisknout primo na
obrazovku.

Promenne instanci:
fName <String> 	 	- jmeno grafu
step <Double>  		- krok zapisu do grafu
output <Integer>  		- typ vypisu
                    0	 - vypis do souboru a na obrazovku
                    1	 - vypis do souboru
                    2	 - vypis na obrazovku
input <Array>  		- pole odkazu na vypisovane veliciny
file <File>  			- odkaz na soubor, do ktereho se zapisuje
graph <Array>   		- pole hodnot zapisovanych do grafu (obrazovka)
graphView <Pointer>  	- odkaz na okno, do ktereho se vypisuje graf
time <Double>           	- cas dalsiho zapisu hodnoty do grafu'!


!Graph methodsFor: 'task language'!

as: oInput 
	"Nastavuje odkazy na vstupni hodnoty."

	input := Array new: 1.
	input at: 1 put: oInput!

as: oInput o: tInput 
	"Nastavuje odkazy na vstupni hodnoty."

	input := Array new: 2.
	input at: 1 put: tInput.
	input at: 2 put: oInput!

as: oInput o: tInput o: thInput 
	"Nastavuje odkazy na vstupni hodnoty."

	input := Array new: 3.
	input at: 1 put: thInput.
	input at: 2 put: tInput.
	input at: 3 put: oInput!

as: oInput o: tInput o: thInput o: fInput 
	"Nastavuje odkazy na vstupni hodnoty."

	input := Array new: 4.
	input at: 1 put: fInput.
	input at: 2 put: thInput.
	input at: 3 put: tInput.
	input at: 4 put: oInput!

file
	"Vypise vsechny hodnoty, ktere se maji zapsat do souboru."

	| x |
	self number: MySimulation active time on: file.
	x := input size + 1.
	[(x := x - 1) > 0]
		whileTrue: [self number: (input at: x) value on: file].
	file cr!

graph
	"Vypise vsechny hodnoty, ktere se maji zapsat do grafu."

	| x t g p |
	t := MySimulation active time.
	x := input size + 1.
	g := Array new: x.
	x := x + 1.
	g at: 1 put: t.
	[(x := x - 1) > 1]
		whileTrue: 
			[g at: x put: (p := (input at: x - 1) value).
			p > 0
				ifTrue: [self max: p]
				ifFalse: [self min: p]].
	graph := graph copyWith: g.
	graph size > 1 ifTrue: [graphView newValue: graph]!

show: oInput 
	"Nastavuje odkazy na vstupni hodnoty."

	input == nil ifTrue: [input := Array new: 0].
	input := input copyWith: oInput!

stop
	"Uzavre vystupni soubor a prekresli graf."

	output = 0 | (output = 1) ifTrue: [file close].
	output = 0 | (output = 2) ifTrue: [graphView endValue]! !

!Graph methodsFor: 'printing'!

firstHeader
	"Pomocna metoda pro vystup grafu - vypisuje prvni textovou 
	hlavicku."

	| x |
	self string: 'cas' on: file.
	x := input size + 1.
	[(x := x - 1) > 0]
		whileTrue: [self string: (input at: x) getName on: file].
	file cr.
	x := input size + 2.
	[(x := x - 1) > 0]
		whileTrue: [file nextPutAll: '--------------'].
	file cr!

number: aNumber on: aStream 
	"Pomocna metoda vypisujici cislo 'aNumber' na 14 mist na vystup 
	'aStream'."

	| myStream |
	myStream := aNumber printString.
	myStream size < 14 ifTrue: [14 - myStream size timesRepeat: [aStream nextPut: $ ]].
	aStream nextPutAll: aNumber printString!

string: aString on: aStream 
	"Pomocna metoda vypisujici retezec 'aString' na 14 mist na vystup 
	'aStream'."

	aString size < 14 ifTrue: [14 - aString size timesRepeat: [aStream nextPut: $ ]].
	aStream nextPutAll: aString!

text
	"Pomocna metoda vraci popis prubehu grafu."

	| x pom |
	x := input size + 1.
	pom := Array new: x - 1.
	[(x := x - 1) > 0]
		whileTrue: [pom at: x put: (input at: x) getName].
	^pom! !

!Graph methodsFor: 'accessing'!

max: aNumber 
	"Nastavuje novou maximalni hodnotu zobrazitelnou v grafu."

	| log basic x num |
	graphView nMax < aNumber
		ifTrue: 
			[num := aNumber.
			aNumber <= 0
				ifTrue: [graphView nMax: 0]
				ifFalse: 
					[log := num asDouble floorLog10.
					num < 1 ifTrue: [log := log - 1].
					basic := 10 ** log.
					x := 0.
					[x * basic < num & (num <= ((x := x + 1) * basic))] whileFalse.
					graphView nMax: (x * basic) asDouble]]!

min: aNumber 
	"Nastavuje novou minimalni hodnotu zobrazitelnou v grafu."

	| log basic x num |
	graphView nMin > aNumber
		ifTrue: 
			[num := aNumber abs.
			aNumber >= 0
				ifTrue: [graphView nMin: 0]
				ifFalse: 
					[log := num asDouble floorLog10.
					num < 1 ifTrue: [log := log - 1].
					basic := 10 ** log.
					x := 0.
					[x * basic < num & (num <= ((x := x + 1) * basic))] whileFalse.
					graphView nMin: (x * basic) negated asDouble]]!

step
	"Vraci krok zaznamu grafu."

	^step!

test
	"Vraci typ vystupu."

	^output!

time
	"Vraci nasledujici cas zaznamu."

	^time!

time: newTime 
	"Nastavuje nasledujici cas zaznamu."

	time := newTime!

view
	"Vraci odkaz na graf okno, kde se zobrazuje graf."

	^graphView! !

!Graph methodsFor: 'fileOut'!

fileOut
	"Nastavi odkaz na soubor, do ktereho se bude ukladat graf."

	| aFile |
	aFile := ''.
	[aFile = '' | (aFile = ' ')]
		whileTrue: [aFile := Filename
						request: 'Zadejte jmeno souboru: '
						initially: (fName chopTo: 8)
								, '.out'
						shouldExist: false].
	^aFile! !

!Graph methodsFor: 'initialization'!

initialize: gName by: stepG 
	"Inicializuje atributy grafu, tj. krok zaznamu 'stepG', jmeno 'fName'."

	stepG <= 0 ifTrue: [MySimulation error: 'Chybne zadany krok v grafu ' , gName].
	fName := gName.
	file := (Filename named: self fileOut) writeStream.
	graph := Array new: 0.
	step := stepG.
	graphView := AreaGraphView
				open: fName
				model: graph
				start: 0
				end: 10.
	output := 0!

initializeFile: gName by: stepG 
	"Inicializuje atributy grafu, tj. krok zaznamu 'stepG', jmeno 'fName'."

	stepG <= 0 ifTrue: [MySimulation error: 'Chybne zadany krok v grafu ' , gName].
	graph := nil.
	fName := gName.
	step := stepG.
	output := 1.
	file := (Filename named: self fileOut) writeStream!

initializeGraph: gName by: stepG 
	"Inicializuje atributy grafu, jmeno 'fName', krok zaznamu 'stepG'."

	stepG <= 0 ifTrue: [MySimulation error: 'Chybne zadany krok v grafu ' , gName].
	file := nil.
	step := stepG.
	graph := Array new: 0.
	fName := gName.
	graphView := AreaGraphView
				open: fName
				model: graph
				start: 0
				end: 10.
	output := 2! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Graph class
	instanceVariableNames: ''!


!Graph class methodsFor: 'instance creation'!

file: gName by: stepG 
	"Vytvori novy graf s nazvem 'gName' a aktivuje metodu pro 
	inicializaci jeho atributu."

	^MySimulation active graphNew: (super new initializeFile: gName by: stepG)!

graph: gName by: stepG 
	"Vytvori novy graf s nazvem 'gName' a aktivuje metodu pro 
	inicializaci jeho atributu."
"
	| active |
	active _ ScheduledControllers activeController.
	(active isKindOf: StandardSystemController)
		ifTrue: [active under]."
	^MySimulation active graphNew: (super new initializeGraph: gName by: stepG)!

new: gName by: stepG 
	"Vytvori novy graf s nazvem 'gName' a aktivuje metodu pro 
	inicializaci jeho atributu."

"	| active |
	active _ ScheduledControllers activeController.
	(active isKindOf: StandardSystemController)
		ifTrue: [active under]."
	^MySimulation active graphNew: (super new initialize: gName by: stepG)!

new: gName by: stepG type: aType 
	"Vytvori novy graf s nazvem 'gName' a aktivuje podle modu, typ 
	zobrazovani."

	| active |
	aType = 0
		ifTrue: 
			["active _ ScheduledControllers activeController.
			(active isKindOf: StandardSystemController)
				ifTrue: [active under]."
			^self new: gName by: stepG]
		ifFalse: [aType = 1
				ifTrue: [^self file: gName by: stepG]
				ifFalse: [aType = 2
						ifTrue: 
							["active _ ScheduledControllers activeController.
							(active isKindOf: StandardSystemController)
								ifTrue: [active under]."
							^self graph: gName by: stepG]
						ifFalse: [MySimulation error: 'Spatne zadany typ grafu' , gName]]]! !

ZaklSpoj subclass: #Qntzr
	instanceVariableNames: 'step eval '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Qntzr comment:
' Trida ''Qntzr'' vytvari funkcni bloky ''Kvantizator'' (nestavove bloky).

Promenne instanci:
step <Double>     - parametr funkcniho bloku
eval <Boolean>    - priznak vyhodnocovani funkcniho bloku'!


!Qntzr methodsFor: 'accessing'!

step
	"Vraci hodnotu kvantizacniho kroku."

	^step! !

!Qntzr methodsFor: 'initialization'!

initialize: newName step: newStep 
	"Inicializuje atributy 'Qntzr'- kvantizator."

	newStep < 0 ifTrue: [MySimulation error: 'Spatne zadany krok kvantizatoru.'].
	step := newStep asDouble.
	eval := false.
	super initialize: newName! !

!Qntzr methodsFor: 'task language'!

value
	"Vraci hodnotu vystupu."

	| pom znam k1 |
	eval ifTrue: [MySimulation error: 'V simulacnim modelu je algebraicka smycka.'].
	eval := true.
	pom := input value asDouble.
	eval := false.
	znam := 1.
	pom < 0
		ifTrue: 
			[znam := -1.
			pom := pom abs].
	pom = 0 ifTrue: [znam := 0].
	step * (pom // step) + (step / 2) <= (pom * 0.999 asDouble)
		ifTrue: [k1 := step * (pom // step) + step]
		ifFalse: [k1 := step * (pom // step)].
	^k1 * znam! !

!Qntzr methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor kvantizatoru."

	SimInspectorView open: ((QntzrInspector inspect: self)
			initialize: simModel)
		label: 'Kvantizator `' , name , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Qntzr class
	instanceVariableNames: ''!


!Qntzr class methodsFor: 'instance creation'!

new: newName step: newStep 
	"Vytvari novou instanci tridy 'Qntzr' a inicializuje ji."

	^super new initialize: newName step: newStep! !

Object subclass: #Tim
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!


!Tim methodsFor: 'accessing'!

getName
	"Vraci jmeno bloku."

	^'Cas'! !

!Tim methodsFor: 'task language'!

value
	"Vraci hodnotu simulacniho casu."

	^MySimulation time! !

SpojStav subclass: #Hyst
	instanceVariableNames: 'yLow yHigh tgAlpha '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Hyst comment:
' Trida ''Hyst'' vytvari funkcni bloky ''Hystereze'' (stavove bloky).
Instancni promenne prebira po svem otci.

Promenne instanci:
yLow <Double>  - parametr funkcniho bloku
yHigh <Double>  - parametr funkcniho bloku
tgAlpha <Double>  - parametr funkcniho bloku'!


!Hyst methodsFor: 'accessing'!

tgAlpha
	"Vraci parametr 'tgAlpha'."

	^tgAlpha!

yHigh
	"Vraci vyssi mez na ose y."

	^yHigh!

yLow
	"Vraci nizsi mez na ose y."

	^yLow! !

!Hyst methodsFor: 'initialization'!

initialize: newName low: newLow high: newHigh yLow: newyLow yHigh: newyHigh tgAlpha: newTgAlpha initVal: newInitVal 
	"Inicializuje atributy 'Hyst' - hystereze."

	newLow >= newHigh ifTrue: [MySimulation error: ' Parametr low>=high u tridy Hyst.'].
	super
		initialize: newName
		low: newLow
		high: newHigh.
	newyLow >= newyHigh ifTrue: [MySimulation error: ' Parametr ylow>=yhigh u tridy Hyst.'].
	yLow := newyLow.
	yHigh := newyHigh.
	tgAlpha := newTgAlpha.
	super initVal: ((newInitVal max: yLow)
			min: yHigh)!

initVal: newInitVal 
	"Nastavi inicializacni hodnotu bloku 'Hyst'."

	super initVal: ((newInitVal max: yLow)
			min: yHigh)! !

!Hyst methodsFor: 'task language'!

eval
	"Nastavuje hodnotu vystupu z hystereze."

	| pom ys yn2 yn1 |
	st := input value asDouble.
	ys := stl.
	yn2 := st - high * tgAlpha.
	yn1 := st - low * tgAlpha.
	pom := ys.
	ys <= yn2 ifTrue: [pom := yn2 min: yHigh].
	ys >= yn1 ifTrue: [pom := yn1 max: yLow].
	st := pom! !

!Hyst methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor hystereze."

	SimInspectorView open: ((HystInspector inspect: self)
			initialize: simModel)
		label: 'Hystereze `' , name , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Hyst class
	instanceVariableNames: ''!


!Hyst class methodsFor: 'instance creation'!

new: newName low: newLow high: newHigh yLow: newyLow yHigh: newyHigh tgAlpha: newTgAlpha 
	"Vytvari a inicializuje blok 'Hyst' - hystereze."

	^super new
		initialize: newName
		low: newLow
		high: newHigh
		yLow: newyLow
		yHigh: newyHigh
		tgAlpha: newTgAlpha
		initVal: 0!

new: newName low: newLow high: newHigh yLow: newyLow yHigh: newyHigh tgAlpha: newTgAlpha initVal: newInitVal 
	"Vytvari a inicializuje blok 'Hyst' - hystereze."

	^super new
		initialize: newName
		low: newLow
		high: newHigh
		yLow: newyLow
		yHigh: newyHigh
		tgAlpha: newTgAlpha
		initVal: newInitVal! !

SpojNestav subclass: #Lim
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Lim comment:
' Trida ''Lim'' vytvari funkcni bloky ''Omezeni'' (nestavove bloky).
Instancni promenne prebira po svem otci.'!


!Lim methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor omezeni."

	SimInspectorView open: ((LimInspector inspect: self)
			initialize: simModel)
		label: 'Omezeni `' , name , '`'! !

!Lim methodsFor: 'task language'!

value
	"Vraci hodnotu vystupu z omezeni."

	| pom |
	eval ifTrue: [MySimulation error: 'V simulacnim modelu je algebraicka smycka.'].
	eval := true.
	pom := input value asDouble.
	eval := false.
	pom > high ifTrue: [^high * tgAlpha].
	pom < low ifTrue: [^low * tgAlpha].
	^pom * tgAlpha! !

!Lim methodsFor: 'initialization'!

initialize: newName low: newLow high: newHigh tgAlpha: newTgAlpha 
	"Inicializuje atributy 'Lim' - omezeni."

	newLow >= newHigh ifTrue: [MySimulation error: ' Parametr low>=high u tridy Lim.'].
	super
		initialize: newName
		low: newLow
		high: newHigh
		tgAlpha: newTgAlpha! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Lim class
	instanceVariableNames: ''!


!Lim class methodsFor: 'instance creation'!

new: newName low: newLow high: newHigh 
	"Vytvari novou instanci tridy 'Lim' a inicializuje ji."

	^super
		new: newName
		low: newLow
		high: newHigh
		tgAlpha: 1!

new: newName low: newLow high: newHigh tgAlpha: newTgAlpha 
	"Vytvari novou instanci tridy 'Lim' a inicializuje ji."

	^super
		new: newName
		low: newLow
		high: newHigh
		tgAlpha: newTgAlpha! !

SpojNestav subclass: #Insv
	instanceVariableNames: 'tgBeta '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Insv comment:
' Trida ''Insv'' vytvari funkcni bloky ''Necitlivost'' (nestavove bloky).
Instancni promenne prebira po svem otci.

Promenne instanci:
tgBeta <Double>  - parametr funkcniho bloku'!


!Insv methodsFor: 'accessing'!

tgBeta
	"Vraci parametr 'tgBeta'."

	^tgBeta! !

!Insv methodsFor: 'initialization'!

initialize: newName low: newLow high: newHigh tgAlpha: newTgAlpha tgBeta: newTgBeta 
	"Inicializuje atributy 'Insv' - necitlivost."

	newLow >= newHigh ifTrue: [MySimulation error: ' Parametr low>=high u tridy Insv.'].
	tgBeta := newTgBeta.
	super
		initialize: newName
		low: newLow
		high: newHigh
		tgAlpha: newTgAlpha! !

!Insv methodsFor: 'task language'!

value
	"Vraci hodnotu vystupu."

	| pom |
	eval ifTrue: [MySimulation error: 'V simulacnim modelu je algebraicka smycka.'].
	eval := true.
	pom := input value asDouble.
	eval := false.
	pom > high ifTrue: [^pom - high * tgAlpha].
	pom < low ifTrue: [^pom - low * tgBeta].
	^0! !

!Insv methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor necitlivosti."

	SimInspectorView open: ((InsvInspector inspect: self)
			initialize: simModel)
		label: 'Necitlivost `' , name , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Insv class
	instanceVariableNames: ''!


!Insv class methodsFor: 'instance creation'!

new: newName low: newLow high: newHigh 
	"Vytvari novou instanci tridy 'Insv' a inicializuje ji."

	^super new
		initialize: newName
		low: newLow
		high: newHigh
		tgAlpha: 1
		tgBeta: 1!

new: newName low: newLow high: newHigh tgAlpha: newTgAlpha tgBeta: newTgBeta 
	"Vytvari novou instanci tridy 'Insv' a inicializuje ji."

	^super new
		initialize: newName
		low: newLow
		high: newHigh
		tgAlpha: newTgAlpha
		tgBeta: newTgBeta! !

SpojStav subclass: #Blash
	instanceVariableNames: 'tgAlpha '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Blash comment:
' Trida ''Blash'' vytvari funkcni bloky ''Vule v prevodech'' (stavove bloky).
Instancni promenne prebira po svem otci.

Promenne instanci:
tgAlpha <Double>  - parametr funkcniho bloku'!


!Blash methodsFor: 'accessing'!

tgAlpha
	"Vraci parametr 'tgAlpha'."

	^tgAlpha! !

!Blash methodsFor: 'initialization'!

initialize: newName low: newLow high: newHigh tgAlpha: newTgAlpha initVal: newInitVal 
	"Inicializuje atributy 'Blash' - vule v prevodech."

	newLow >= newHigh ifTrue: [MySimulation error: ' Parametr low>=high u tridy Blash.'].
	super
		initialize: newName
		low: newLow
		high: newHigh.
	tgAlpha := newTgAlpha asDouble.
	super initVal: (newInitVal max: newHigh)!

initVal: newInitVal 
	"Nastavi inicializacni hodnotu bloku 'Blash'."

	super initVal: (newInitVal max: high)! !

!Blash methodsFor: 'task language'!

eval
	"Nastavuje hodnotu vystupu z vule v prevodech."

	| ys yn1 yn2 y |
	st := input value asDouble.
	ys := stl.
	yn1 := st - low * tgAlpha.
	yn2 := st - high * tgAlpha.
	y := yn2 max: ys.
	y := yn1 min: y.
	st := y! !

!Blash methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor vule v prevodech."

	SimInspectorView open: ((BlashInspector inspect: self)
			initialize: simModel)
		label: 'Vule v prevodech `' , name , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Blash class
	instanceVariableNames: ''!


!Blash class methodsFor: 'instance creation'!

new: newName low: newLow high: newHigh tgAlpha: newTgAlpha 
	"Vytvari a inicializuje blok 'Blash' - vule v prevodech."

	^super new
		initialize: newName
		low: newLow
		high: newHigh
		tgAlpha: newTgAlpha
		initVal: newHigh!

new: newName low: newLow high: newHigh tgAlpha: newTgAlpha initVal: newInitVal 
	"Vytvari a inicializuje blok 'Blash' - vule v prevodech."

	^super new
		initialize: newName
		low: newLow
		high: newHigh
		tgAlpha: newTgAlpha
		initVal: newInitVal! !

SpojNestav subclass: #Frict
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Frict comment:
' Trida ''Frict'' vytvari funkcni bloky ''Treni'' (nestavove bloky).
Instancni promenne prebira po svem otci.'!


!Frict methodsFor: 'task language'!

value
	"Vraci hodnotu vystupu z treni."

	| pom |
	eval ifTrue: [MySimulation error: 'V simulacnim modelu je algebraicka smycka.'].
	eval := true.
	pom := input value asDouble.
	eval := false.
	pom > 0 ifTrue: [^high + (pom * tgAlpha)].
	pom < 0 ifTrue: [^low + (pom * tgAlpha)].
	^0! !

!Frict methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor treni."

	SimInspectorView open: ((FrictInspector inspect: self)
			initialize: simModel)
		label: 'Treni `' , name , '`'! !

!Frict methodsFor: 'initialization'!

initialize: newName low: newLow high: newHigh tgAlpha: newTgAlpha 
	"Inicializuje atributy 'Frict' - treni."

	newLow >= newHigh ifTrue: [MySimulation error: ' Parametr low>=high u tridy Frict.'].
	super
		initialize: newName
		low: newLow
		high: newHigh
		tgAlpha: newTgAlpha! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Frict class
	instanceVariableNames: ''!


!Frict class methodsFor: 'instance creation'!

new: newName low: newLow high: newHigh 
	"Vytvari novou instanci tridy 'Frict' a inicializuje ji."

	^super
		new: newName
		low: newLow
		high: newHigh
		tgAlpha: 1!

new: newName low: newLow high: newHigh tgAlpha: newTgAlpha 
	"Vytvari novou instanci tridy 'Frict' a inicializuje ji."

	^super
		new: newName
		low: newLow
		high: newHigh
		tgAlpha: newTgAlpha! !

ZaklSpoj subclass: #Integrator
	instanceVariableNames: 'ss ssl dd ddl initVal '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BasicSimulationCont'!
Integrator comment:
' Trida ''Integrator'' vytvari funkcni bloky ''Integrator''.

Promenne instanci:
ss <Double>  - hodnota vstupu integratoru
ssl <Double>  - hodnota vstupu integratoru stara
dd <Double>  - hodnota vystupu integratoru
ddl <Double>  - hodnota vystupu integratoru stara
initVal <Double>  - parametr funkcniho bloku inicializacni hodnota'!


!Integrator methodsFor: 'accessing'!

dd
	"Vraci hodnotu promenne 'dd'."

	^dd!

dd: newdd 
	"Nastaveni promenne 'dd'."

	dd := newdd asDouble!

ddl
	"Vraci hodnotu promenne 'ddl'."

	^ddl!

ddl: newddl 
	"Nastaveni promenne 'ddl'."

	ddl := newddl asDouble!

init
	"Vraci hodnotu promenne 'initValue'."

	^initVal!

lTon
	"Obnova stavu integratoru."

	ss := ssl.
	dd := ddl!

nTol
	"Uschova stavu integratoru."

	ssl := ss.
	ddl := dd!

ss
	"Vraci hodnotu promenne 'ss'."

	^ss!

ss: newss 
	"Nastaveni promenne 'ss'."

	ss := newss asDouble!

ssl
	"Vraci hodnotu promenne 'ssl'."

	^ssl!

ssl: newssl 
	"Nastaveni promenne 'ssl'."

	ssl := newssl asDouble! !

!Integrator methodsFor: 'initialization'!

initialize: newName 
	"Inicializuje integrator."

	super initialize: newName.
	ss := 0.
	ssl := 0.
	dd := 0.
	ddl := 0.
	initVal := 0.
	MySimulation active reset: true!

initialize: newName initVal: newInitVal 
	"Inicializuje integrator."

	super initialize: newName.
	ssl := 0.
	dd := 0.
	ddl := 0.
	self initVal: newInitVal!

initVal
	"Inicializace hodnoty pro integrator."

	self initVal: initVal.
	dd := 0!

initVal: newValue 
	"Inicializace hodnoty pro integrator."

	ss := newValue asDouble.
	initVal := ss asDouble.
	MySimulation active reset: true! !

!Integrator methodsFor: 'task language'!

eval
	"Vyhodnoceni vstupu integratoru."

	dd := input value asDouble!

setValue: newValue 
	"Nastavuje aktualni hodnotu integratoru."

	ss := newValue asDouble.
	MySimulation active reset: true!

value
	"Vrati stav integratoru."

	^ss! !

!Integrator methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor integratoru."

	SimInspectorView open: ((IntgInspector inspect: self)
			initialize: simModel)
		label: 'Integrator `' , name , '`'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Integrator class
	instanceVariableNames: ''!


!Integrator class methodsFor: 'instance creation'!

new: newName 
	"Vytvari novou instanci tridy 'Integrator' a inicializuje ji."

	^MySimulation active integrNew: (super new initialize: newName)!

new: newName initVal: newInitVal 
	"Vytvari novou instanci tridy 'Integrator' a inicializuje ji."

	^MySimulation active integrNew: (super new initialize: newName initVal: newInitVal)! !