Object subclass: #Atribut
	instanceVariableNames: 'atributValue defaultValue lock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!Atribut methodsFor: 'initialize'!

initialize
	"inicializace atributu"

	atributValue _ nil.
	defaultValue _ 0.
	lock _ false.
	^self! !

!Atribut methodsFor: 'locking'!

lock
	"zmkne atribut. Jeho hodnota nesmi byt zmenena"

	lock _ true!

unlock
	"odemkne atribut. Jeho hodnota muze byt zmenena"

	lock _ false! !

!Atribut methodsFor: 'access'!

atributValue
	"vraci hodnotu atributu"

	atributValue == nil
		ifTrue: [^defaultValue]
		ifFalse: [^atributValue]!

atributValue: aValue 
	"nastavi hodnotu atributu"

	lock
		ifTrue: [self error: 'Zapis do zamceneho atributu']
		ifFalse: [atributValue _ aValue]!

clear
	"Navrat k implicitni hodnote"

	atributValue _ nil!

defaultValue
	"vraci default hodnotu atributu"

	^defaultValue!

defaultValue: aValue
	"meni default hodnotu atributu"

	defaultValue _ aValue!

isLocked
	"vraci logickou hodnotu nastaveni zamku"

	^lock! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Atribut class
	instanceVariableNames: ''!


!Atribut class methodsFor: 'instance creation'!

defaultValue: defVal lock: bool 
	"vytvori novy atribut a nastavi hodnoty defaultValue a lock"

	| newAtribute |
	newAtribute _ self new.
	newAtribute defaultValue: defVal.
	bool
		ifTrue: [newAtribute lock]
		ifFalse: [newAtribute unlock].
	^newAtribute!

new
	"vytvari novy atribut"

	^super new initialize! !

LinearFit subclass: #OrientedLinearFit
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!OrientedLinearFit methodsFor: 'transformations'!

makeOrthogonal
	| dx dy |
	1 to: self size - 1 do: 
		[:i | 
		dx _ ((self at: i) x - (self at: i + 1) x) abs.
		dy _ ((self at: i) y - (self at: i + 1) y) abs.
		dx < 5 ifTrue: [self at: i + 1 put: (self at: i) x @ (self at: i + 1) y].
		dy < 5 ifTrue: [self at: i + 1 put: (self at: i + 1) x @ (self at: i) y]]!

moveBeginBy: offset
	"posune zacatek lomene cary o offset"

	self at: 1 put: ((self at: 1) translateBy: offset).
	^self!

moveBeginTo: aPoint
	"posune zacatek lomene cary do bodu aPoint"

	self at: 1 put: aPoint.
	^self!

moveEndBy: offset
	"posune konec lomene cary o offset"

	self at: (self size) put: ((self at: (self size)) translateBy: offset).
	^self!

moveEndTo: aPoint
	"posune konec lomene cary do bodu aPoint"

	self at: (self size) put: aPoint.
	^self! !

!OrientedLinearFit methodsFor: 'displaying'!

displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger mask: aForm 
	"Display the receiver on the display medium aDisplayMedium positioned at 
	aDisplayPoint within 
	the rectangle clipRectangle and with the rule, ruleInteger, and mask, aForm."

	| arrow line |
	super
		displayOn: aDisplayMedium
		at: aDisplayPoint
		clippingBox: clipRectangle
		rule: ruleInteger
		mask: aForm.
	arrow _ self countArrow.
	line _ LinearFit new.
	line form: self form.
	line add: (arrow at: 1).
	line add: (self at: self size).
	line add: (arrow at: 2).
	line
		displayOn: aDisplayMedium
		at: aDisplayPoint
		clippingBox: clipRectangle
		rule: ruleInteger
		mask: aForm! !

!OrientedLinearFit methodsFor: 'private'!

clipLine: aLine By: aRect 
	"orizne caru tak, aby lezela vne obdelnika aRect"

	| dx dy k xx yy x1 y1 pn |

	(aRect containsPoint: aLine beginPoint)
		& ((aRect containsPoint: aLine endPoint) not)
		ifTrue: 
			[dx _ (aLine endPoint - aLine beginPoint) x.
			dy _ (aLine endPoint - aLine beginPoint) y.
			x1 _ aLine beginPoint x.
			y1 _ aLine beginPoint y.
			dx > 0
				ifTrue: 
					["prusecik s pravou stranou obdelnika"
					k _ dy / dx.
					yy _ y1 + (k * (aRect right - x1)).
					pn _ aRect right @( yy rounded).
					((yy >= aRect top) & (yy <= aRect bottom))
						ifTrue: 
							[aLine beginPoint: pn.
							^aLine]].
			dx < 0
				ifTrue: 
					["prusecik s levou stranou obdelnika"
					k _ dy / dx.
					yy _ y1 + (k * (aRect left - x1)).
					pn _ aRect left @ (yy rounded).
					((yy >= aRect top) & (yy <= aRect bottom))
						ifTrue: 
							[aLine beginPoint: pn.
							^aLine]].
			dy < 0
				ifTrue: 
					["prusecik s horni stranou obdelnika"
					k _ dx / dy.
					xx _ (x1 + (k * (aRect top - y1))) rounded.
					pn _ xx @ aRect top.
					((xx >= aRect left) & (xx <= aRect right))
						ifTrue: 
							[aLine beginPoint: pn.
							^aLine]].
			dy > 0
				ifTrue: 
					["prusecik s dolni stranou obdelnika"
					k _ dx / dy.
					xx _ (x1 + (k * (aRect bottom - y1))) rounded.
					pn _ xx @ aRect bottom.
					((xx >= aRect left) & (xx <= aRect right))
						ifTrue: 
							[aLine beginPoint: pn.
							^aLine]]].
	^aLine!

countArrow
	"vypocita souradnice sipky"

	| points s dx dy k xa ya p1 p2 la |
	points _ Array new: 2.
	s _ self size.
	s < 2
		ifFalse: 
			[p1 _ self at: s - 1.
			p2 _ self at: s.
			dx _ p2 x - p1 x.
			dy _ p2 y - p1 y.
			dx abs > dy abs
				ifTrue: 
					[k _ 0 - (dy / dx).
					la _ dx > 0 ifTrue: [10] ifFalse: [-10].
					xa _ p2 x - la.
					ya _ p2 y + (k + 0.2 * la).
					points at: 1 put: xa @ ya.
					ya _ p2 y + (k - 0.2 * la).
					points at: 2 put: xa @ ya]
				ifFalse: 
					[k _ 0 - (dx / dy).
					la _ dy > 0 ifTrue: [10] ifFalse: [-10].
					ya _ p2 y - la.
					xa _ p2 x + (k + 0.2 * la).
					points at: 1 put: xa @ ya.
					xa _ p2 x + (k - 0.2 * la).
					points at: 2 put: xa @ ya].
			^points]! !

!OrientedLinearFit methodsFor: 'clipping'!

clipBeginBy: aRect
	"orizne zacatek lomene cary vne obdelnika aRect"

	| ln|
	ln _ Line new.
	ln beginPoint: (self at: 1).
	ln endPoint: (self at: 2).
	ln _ self clipLine: ln By: aRect.
	self at:1 put: (ln beginPoint).
	^self!

clipEndBy: aRect
	"orizne zacatek lomene cary vne obdelnika aRect"

	| ln s|
	ln _ Line new.
	s _ self size.
	ln beginPoint: (self at: s).
	ln endPoint: (self at: (s-1)).
	ln _ self clipLine: ln By: aRect.
	self at: s put: (ln beginPoint).
	^self! !

!OrientedLinearFit methodsFor: 'accessing'!

distanceFrom: aPoint 
	"vraci vzdalenost lomenne cary od bodu aPoint"

	| minDist pp d p1 p2 dx dy u |
	minDist _ nil.
	1 to: self size - 1 do: 
		[:i | 
		p1 _ self at: i.
		p2 _ self at: i + 1.
		pp _ aPoint nearestIntegerPointOnLineFrom: p1 to: p2.
		dx _ p2 x - p1 x.
		dy _ p2 y - p1 y.
		dx ~= 0
			ifTrue: [u _ pp x - p1 x / dx]
			ifFalse: [u _ pp y - p1 y / dy].
		u < 0 ifTrue: [pp _ p1].
		u > 1 ifTrue: [pp _ p2].
		d _ aPoint dist: pp.
		minDist == nil
			ifTrue: [minDist _ d]
			ifFalse: [d < minDist ifTrue: [minDist _ d]]].
	^minDist!

pointNearestTo: aPoint 
	| minDist inx dd |
	minDist _ nil.
	inx _ 1.
	1 to: self size do: 
		[:i | 
		dd _ (self at: i)
					dist: aPoint.
		minDist == nil
			ifTrue: 
				[minDist _ dd.
				inx _ i]
			ifFalse: [minDist > dd
					ifTrue: 
						[minDist _ dd.
						inx _ i]]].
	inx = 1 | (inx = self size)
		ifTrue: [^nil]
		ifFalse: [^inx]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OrientedLinearFit class
	instanceVariableNames: ''!


!OrientedLinearFit class methodsFor: 'examples'!

linearFitSample
	"Select points on a Path using the red button.  Terminate by selecting any other
	button.  Creates a Path from the points and displays it as a piece-wise linear
	approximation." 

	"OrientedLinearFit linearFitSample."

	| aLinearFit aForm flag |
	aLinearFit _ OrientedLinearFit new.
	aForm _ Form new extent: 2@2.
	aForm  black.
	aLinearFit form: aForm.
	flag _ true.
	[flag] whileTrue:
		[Sensor waitButton.
		 Sensor redButtonPressed
			ifTrue: [aLinearFit add: Sensor waitButton. Sensor waitNoButton.
					aForm displayOn: Display at: aLinearFit last]
			ifFalse: [flag_false]].
	aLinearFit displayOn: Display! !

OrientedLinearFit subclass: #Connection
	instanceVariableNames: 'startNode endNode label labelPos atributes '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!Connection methodsFor: 'private'!

countLabelPos
	"Vypocita optimalni polohu labelu."

	| dd |
	dd _ self secondPoint - self firstPoint.
	dd x > 0 & (dd y > 0) | (dd x < 0 & (dd y < 0)) ifTrue: [dd _ dd translateBy: 10@-40].
	labelPos _ (dd / 2) rounded! !

!Connection methodsFor: 'displaying'!

displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger mask: aForm 
	"Zobrazi spoj na aDisplayMedimu na pozici aDisplayPoint. Label je zobrazen na 
	realivni pozici odvozene od prvniho bodu."

	| rule |
	super
		displayOn: aDisplayMedium
		at: aDisplayPoint
		clippingBox: clipRectangle
		rule: ruleInteger
		mask: aForm.
	labelPos == nil ifTrue: [self countLabelPos].
	rule _ ruleInteger.
	ruleInteger = Form over ifTrue: [rule _ Form under].
	label asParagraph
		displayOn: aDisplayMedium
		at: aDisplayPoint + labelPos + self firstPoint
		clippingBox: clipRectangle
		rule: rule
		mask: aForm! !

!Connection methodsFor: 'accessing'!

begin
	"vraci index uzelu ze ktereho vychazi"

	^startNode!

begin: aNodeIndex 
	"zapise index pocatecniho uzlu"

	startNode _ aNodeIndex!

end
	"vraci index ciloveho uzlu"

	^endNode!

end: aNodeIndex
	"zapise index ciloveho uzlu"

	endNode _ aNodeIndex!

label
	"vraci label hrany"

	^label!

label: aString
	"nastavi label hrany"

	label _ aString!

labelPosition
	"vraci relativni pozici labelu vzhledem k hrane"

	labelPos == nil ifTrue: [^0 @ 0].
	^labelPos!

labelPosition: aPoint
	"nastavi relativni pozici labelu vzhledem k hrane"

	labelPos _ aPoint! !

!Connection methodsFor: 'initialize'!

initialize
	self begin: 0.
	self end: 0.
	form _ Form new extent: 2 @ 2.
	form black.
	label _ ''.
	labelPos _ nil.
	atributes _ Dictionary new.
	^self! !

!Connection methodsFor: 'transformation'!

translateBy: aPoint
	"transformuje caru podle bodu"

	|con|
	con _ super translateBy: aPoint.
	con end: (self end).
	con begin: (self begin).
	con label: (self label).
	con labelPosition: labelPos.
	con atributes: atributes.
	^con! !

!Connection methodsFor: 'inspection'!

defaultInspectorClass
	^NodeInspector! !

!Connection methodsFor: 'atributes'!

addAtribute: anAtribute at: key
	"prida do slovniku atributu polozku s klicem key"

	atributes at: key put: anAtribute!

atributeAt: key
	"vraci atribut s klicem key"

	^atributes at: key!

atributes
	"vraci slovnik atributu"

	^atributes!

atributes: aDictionary
	"nastavuje slovnik atributu"

	atributes _ aDictionary!

removeAtribute: key
	"zrusi polozku ve slovniku atriburu s klicem key"

	atributes removeKey: key! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Connection class
	instanceVariableNames: ''!


!Connection class methodsFor: 'instance-creations'!

new
	^super new initialize! !

DisplayObject subclass: #GraphicNode
	instanceVariableNames: 'origin corner number label atributes nInputs nOutputs inputs outputs lastMessageNumber '
	classVariableNames: 'Counter '
	poolDictionaries: ''
	category: 'Graph-componets'!
GraphicNode comment:
'Trida GraphNode predstavuje strukuru, ktera muze reprezentovat uzel v grafu. Jedna se o obdelnik, ktery je oramovan rameckem tloustky 1. Subtridy mohou zmenit vzhled uzlu predefinovanim metody getImage.

Instance obsahuje tyto promene:
  origin  <aPoint>     levy horni roh obdelnika
  corner <aPoint>     pravy dolni roh obdelnika '!


!GraphicNode methodsFor: 'testing'!

containsPoint: aPoint
	"odpovi zda bod aPoint lezi uvnitr obdelnika, ktery  ohranicuje uzel"

	^origin <= aPoint and: [aPoint < corner]!

intersects: aRectangle 
	"Odpovi, zda obdelnik aRectangle protina uzel."

	^(origin max: aRectangle origin) < (corner min: aRectangle corner)! !

!GraphicNode methodsFor: 'compiling'!

checkInOuts
	"zkrontroluje pocty vstupu a vystupu v uzlu"

	nInputs < self minInputs ifTrue: [^self confirm: 'Malo vstupu v uzlu s cislem ' , number printString].
	nOutputs < self minOutputs ifTrue: [^self confirm: 'Malo vystupu v uzlu s cislem ' , number printString].
	^true!

compile
	"Kompilacni metoda uzlu. V odvozenych uzlech bude tato metoda predefinovana."

	self checkInOuts ifFalse: [^false].
	^self syntax!

initCompilation
	"Inicializuje uzel pred kompilaci"

	| sb |
	sb _ [:x1 :x2 | (x1 isKindOf: DataNode)
				& (x2 isKindOf: CommandNode) | ((x1 isKindOf: CommandNode)
					& (x2 isKindOf: CommandNode) | ((x1 isKindOf: DataNode)
						& (x2 isKindOf: DataNode)) & (x1 number < x2 number))].
	inputs _ SortedCollection sortBlock: sb.
	outputs _ SortedCollection sortBlock: sb.
	nInputs _ 0.
	nOutputs _ 0.
	lastMessageNumber _ -1!

initSimulation
	"Inicializuje uzel pred spustenim simulace"

	^self!

next
	^outputs at: 1!

syntax
	"kontroluje syntaxi grafu v okoli uzlu. Tato metoda je specificka pro jednotlive typy uzlu. Musi byt predefinovana."

	^self subclassResponsibility! !

!GraphicNode methodsFor: 'fileIn/Out'!

storeBinaryOn: stream

	stream storeRecordOfClass: self class.
	stream nextPut: origin.
	stream nextPut: corner.
	stream nextPut: self label.
	stream nextPut: self number.
	stream nextPut: self atributes.! !

!GraphicNode methodsFor: 'atributes'!

addAtribute: anAtribute at: key
	"prida do slovniku atributu polozku s klicem key"

	atributes at: key put: anAtribute!

atribute: key
	"vraci polozku ve slovniku atributu s klicem key"

	atributes at: key!

atributeAt: key
	"vraci atribut s klicem key"

	^atributes at: key!

atributes
	"vraci slovnik atributu"

	^atributes!

atributes: aDictionary
	"nastavuje slovnik atributu"

	atributes _ aDictionary!

removeAtribute: key
	"zrusi polozku ve slovniku atriburu s klicem key"

	atributes removeKey: key! !

!GraphicNode methodsFor: 'accessing'!

addInput: n 
	"Zaradi do pole vstupnich bodu novy prvek. N je cislo uzlu, ze ktereho vstup 
	prichazi."

	self maxInputs > nInputs
		ifTrue: 
			["pocet vstupu je omezen"
			nInputs _ nInputs + 1.
			inputs add: n.
			^true]
		ifFalse: 
			[self confirm: 'Prilis mnoho vstupu uzlu ', (number printString).
			^false]!

addOutput: n 
	"Zaradi do pole vystupnich bodu novy prvek. N je cislo uzlu, do ktereho vystup 
	smeruje."

	self maxOutputs > nOutputs
		ifTrue: 
			["pocet vystupu je omezen"
			nOutputs _ nOutputs + 1.
			outputs add: n.
			^true]
		ifFalse: 
			[self confirm: 'Prilis mnoho vystupu uzlu ', (number printString).
			^false]!

corner: aPoint
	"nastavi pravy dolni roh obdelnika"

	corner _ aPoint.!

label
	"vraci retezec s nazvem uzlu"

	^label!

label: aString
	"nastavi novy nazev uzlu"

	label _ aString!

number
	"vraci cislo uzlu"

	^number!

number: aNumber
	"nastavuje novou hodnotu cisla uzlu"

	number _ aNumber!

offset
	"Vraci realtivni posunuti uzlu."

	^origin!

offset: aPoint 
	"Nastavi posunuti uzlu."

	|tempExtent|
	tempExtent _ self extent.
	origin _ aPoint.
	corner _ origin + tempExtent!

origin: aPoint
	"nastavi levy horni roh obdelnika"

	origin _ aPoint.!

relativeRectangle
	"Vraci obdelnik, ktery ma stejny offset jako uzel a stejnou vysku a sirku."
	
	^Rectangle origin: origin corner: corner.! !

!GraphicNode methodsFor: 'display box access'!

computeBoundingBox
	"Vraci obdelnikovou oblast, ve ktere se uzel nachazi."

	^Rectangle origin: 0 @ 0 corner: (corner - origin)! !

!GraphicNode methodsFor: 'displaying'!

displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger mask: aForm 
	self getImage
		displayOn: aDisplayMedium
		at: aDisplayPoint 
		clippingBox: clipRectangle
		rule: ruleInteger
		mask: aForm! !

!GraphicNode methodsFor: 'initialize-release'!

initialize
	"Inicializuje promenne nove vytvorene instance."

	label _ ''.
	ModelarDocument active == nil
		ifTrue: [number _ 0]
		ifFalse: 
			[number _ ModelarDocument active topDocument lastNodeNumber + 1.
			 ModelarDocument active topDocument lastNodeNumber: number].
	nInputs _ nOutputs _ 0.
	self maxInputs > 0 ifTrue: [inputs _ Array new: self maxInputs].
	self maxOutputs > 0 ifTrue: [outputs _ Array new: self maxOutputs].
	atributes _ Dictionary new.
	lastMessageNumber _ -1.
	^self! !

!GraphicNode methodsFor: 'private'!

deepCopy
	inputs _ nil.
	outputs _ nil.
	^super deepCopy!

getImage
	"vraci formu odpovidajici tvaru uzlu"

	| aForm para str |
	aForm _ Form extent: self extent.
	aForm offset: self offset.
	aForm white.
	aForm borderWidth: 1.
	str _ number printString , '.'.
	para _ str asParagraph textStyle: (TextStyle styleNamed: #icon).
	para compositionRectangle: (0 @ 0 corner: self width - 2 @ (self height // 3)).
	para centered.
	para
		displayOn: aForm
		at: 1 @ 1
		clippingBox: self boundingBox
		rule: Form over
		mask: Form black.
	para _ label asParagraph textStyle: (TextStyle styleNamed: #icon).
	para compositionRectangle: (0 @ 0 corner: self width - 2 @ (self height * 0.6)).
	para centered.
	para
		displayOn: aForm
		at: 1 @ (self height // 3 + 2)
		clippingBox: self boundingBox
		rule: Form over
		mask: Form black.
	^aForm! !

!GraphicNode methodsFor: 'inspection'!

defaultInspectorClass
	"Vraci tridu inspectoru, ktery bude implicitne pouzit pro prohlizeni uzlu"

	^NodeInspector! !

!GraphicNode methodsFor: 'error handling'!

confirm: aMessageString 
	"predefinovana metoda ze tridy Object. Zobrazi okno se zpravou 
	aMessageString a ceka na potvrzeni stiskem tlacitka Ok. Pote vraci false."

	ConfirmDialog message: self number printString, ': ', aMessageString.
	^false! !

!GraphicNode methodsFor: 'ioLimits'!

maxInputs
	"vraci maximalni pripustny pocet vstupu uzlu"

	^10!

maxOutputs
	"vraci maximalni pripustny pocet vystupu uzlu"

	^10!

minInputs
	"vraci minimalni pripustny pocet vstupu uzlu"

	^0!

minOutputs
	"vraci minimalni pripustny pocet vystupu uzlu"

	^0! !

!GraphicNode methodsFor: 'message handling'!

broadcast: aMessageSymbol withNumber: aNumber 
	"Zajisti rozsireni zpravy po celem simulacnim modelu. K rozsireni se pouziva 
	metoda zaplavy. Pro rozliseni, zda dana zprava jiz jednou do uzlu prisla se 
	pouziva cislo aNumber."

	aNumber = lastMessageNumber
		ifFalse: 
			[(self respondsTo: aMessageSymbol)
				ifTrue: [self perform: aMessageSymbol].
			lastMessageNumber _ aNumber.
			1 to: nOutputs do: [:i | (outputs at: i)
					broadcast: aMessageSymbol withNumber: aNumber]]!

broadcast: aMessageSymbol withNumber: aNumber from: aSender 
	"Zajisti rozsireni zpravy po celem simulacnim modelu. K rozsireni se pouziva 
	metoda zaplavy. Pro rozliseni, zda dana zprava jiz jednou do uzlu prisla se 
	pouziva cislo aNumber."

	aNumber = lastMessageNumber
		ifFalse: 
			[(self respondsTo: aMessageSymbol)
				ifTrue: [self perform: aMessageSymbol].
			lastMessageNumber _ aNumber.
			1 to: nOutputs do: [:i | (outputs at: i)
					broadcast: aMessageSymbol withNumber: aNumber from: aSender]]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GraphicNode class
	instanceVariableNames: ''!


!GraphicNode class methodsFor: 'instance creation'!

new: aPoint 
	"vytvori novy uzel s nulovym ofsetem a velikosti aPoint"

	|nNode|
	nNode _ self new.
	nNode origin: 0@0.
	nNode corner: aPoint.
	^nNode initialize!

readBinaryFrom: stream tag: tag
	"Vraci object precteny ze streamu"

	|  gn|
	gn _ self new.
	stream register: gn id: tag.
	gn initialize.
	gn origin: stream next.
	gn corner: stream next.
	gn label: stream next.
	gn number: stream next.
	gn atributes: stream next.
	^gn! !

GraphicNode subclass: #Port
	instanceVariableNames: 'externNumber direction '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!
Port comment:
'       Instance tridy Port maji za ukol zprostredkovavat styk mezi hierarchickymi urovnemi vytvareneho modelu.  Maji podobne vlastnosti jako ostatni uzly. Lisi se pouze zpusobem vykresleni - vnitrek portu ma svetle sedou barvu. Misto sveho cisla uvadi cislo uzlu na vyssi urovni na ktery je pripojen.
	Nelze smazat na urovni na ktere se nachazi, ale pouze tak, ze smazeme spoj na strukturovany uzel, kteremu nalezi.

Promenne instance:
	externNumber  <integer>'!


!Port methodsFor: 'ioLimits'!

maxInputs
 ^1!

maxOutputs
 ^1! !

!Port methodsFor: 'initialize-release'!

initialize
	"inicializuje nove vytvoreny port"

	super initialize.
      externNumber _ 0.
	number _ 0 - number.
	^self! !

!Port methodsFor: 'private'!

getImage
	"vraci formu odpovidajici tvaru portu. Je to obdelnik s rameckem tloustky 1 s
	vepsanym cislem uzlu na vyssi urovni, na ktery zprostredkovava spojeni."

	| aForm para str |
	aForm _ Form extent: self extent.
	aForm offset: self offset.
	aForm veryLightGray.
	aForm borderWidth: 1.
	str _ externNumber printString , '.'.
	para _ str asParagraph.
	para compositionRectangle: (0 @ 0 corner: self width - 2 @ (self height - 2)).
	para centered.
	para
		displayOn: aForm
		at: 1 @ 1
		clippingBox: self boundingBox
		rule: Form under
		mask: Form black.
	^aForm! !

!Port methodsFor: 'compiling'!

initConnections
	^self!

initSimulation
	^self!

syntax
	^true! !

!Port methodsFor: 'accessing'!

externNumber
	"vraci cislo externiho spoje pripojenoho na port. Jedna se o index v seznamu
	spoju nadrazeneho dokumentu"

	^externNumber!

externNumber: anIndex 
	"nastavi cislo externiho spoje pripojenoho na port. Jedna se o index v seznamu
	spoju nadrazeneho dokumentu"

	externNumber _ anIndex.
	self changed!

in
	"nastavi uzel jako vstupni"

	direction _ #in!

out
	"nastavi uzel jako vystupni"

	direction _ #out! !

!Port methodsFor: 'fileIn/Out'!

storeBinaryOn: stream 
	super storeBinaryOn: stream.
	stream nextPut: externNumber! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Port class
	instanceVariableNames: ''!


!Port class methodsFor: 'instance creation'!

readBinaryFrom: stream tag: tag 
	| port |
	port _ super readBinaryFrom: stream tag: tag.
	port externNumber: stream next.
	^port! !

GraphicNode subclass: #MainNode
	instanceVariableNames: 'simulation registers '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!MainNode methodsFor: 'ioLimits'!

maxInputs
	^0!

maxOutputs
	^20!

minInputs
	^0!

minOutputs
	^1! !

!MainNode methodsFor: 'simulation'!

initSimulation
	"inicializuje hlavni uzel simulace"

	simulation _ MySimulation new activate.
	simulation init: (self atributeAt: 'Start time') atributValue to: (self atributeAt: 'End time') atributValue.
	registers _ Dictionary new!

run
	"Spusti simulaci modelu, ve kterem se nachazi"

	
	self broadcast: #initSimulation withNumber: 0 - lastMessageNumber.
	Transcript show: 'Spusten simulacni model `' , ModelarDocument active topDocument title , '` ...'; cr; endEntry.
	simulation run; output: Transcript.
	Transcript endEntry!

runOnBackGround
	"Spusti simulaci modelu, ve kterem se nachazi"

	| process  |
	self broadcast: #initSimulation withNumber: 0 - lastMessageNumber.
	Transcript show: 'Na pozadi spusten simulacni model `' , ModelarDocument active topDocument title , '` ...'; cr; endEntry.
	process := 
			[simulation run; output: Transcript.
			Transcript endEntry] newProcess.
	MySimulation addProcess: process with: simulation! !

!MainNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	self addAtribute: (Atribut defaultValue: 0 lock: false)
		at: 'Start time'.
	self addAtribute: (Atribut defaultValue: 1000 lock: false)
		at: 'End time'.
	registers _ Dictionary new.
	label _ 'Hlavni uzel'.
	simulation _ nil.
	^self! !

!MainNode methodsFor: 'compiling'!

registerNode: aNode 
	"zaregistruje uzel aNode v hlavnim uzlu"

	registers at: aNode label put: aNode data!

syntax
	"Zkontroluje zda vsichni naslednici hlavniho uzlu jsou odvozeni od tridy 
	generator"

	(outputs detect: [:node | (node isKindOf: GeneratorNode ) not]
		ifNone: [nil])
		== nil ifTrue: [^true].
	^self confirm: 'Hlavni uzel simulace musi byt spojen pouze s uzly typu generator'! !

!MainNode methodsFor: 'accessing'!

registers
	"vraci slovnik zaregistrovanych uzlu"

	^registers!

simulation
	"vraci hodnotu promenne 'simulation'."

	^simulation! !

!MainNode methodsFor: 'inspection'!

defaultInspectorClass
	^MainNodeInspector! !

MainNode subclass: #ContMainNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!ContMainNode methodsFor: 'compiling'!

initConnections
	^self!

initSimulation
	"inicializuje hlavni uzel simulace"

	simulation _ MySimulation new activate.
	simulation init: (self atributeAt: 'Start time') atributValue to: (self atributeAt: 'End time') atributValue.
	simulation setAccuracy: (self atributeAt: 'Abs error') atributValue relError: (self atributeAt: 'Rel error') atributValue.
	simulation setStep: (self atributeAt: 'Min step') atributValue to: (self atributeAt: 'Max step') atributValue.
	simulation setMethod: (self atributeAt: 'Method') atributValue.
	registers _ Dictionary new!

syntax
	^true! !

!ContMainNode methodsFor: 'ioLimits'!

maxInputs
	^0!

maxOutputs
	^0!

minInputs
	^0!

minOutputs
	^0! !

!ContMainNode methodsFor: 'simulation'!

run
	simulation run!

runOnBackGround
	"Spusti simulaci modelu, ve kterem se nachazi"

	| process |
	Transcript show: 'Na pozadi spusten simulacni model `' , ModelarDocument active topDocument title , '` ...'; cr; endEntry.
	process := 
			[simulation run; output: Transcript.
			Transcript endEntry] newProcess.
	MySimulation addProcess: process with: simulation! !

!ContMainNode methodsFor: 'initialize'!

initialize
	super initialize.
	self addAtribute: (Atribut defaultValue: 0 lock: false)
		at: 'Abs error'.
	self addAtribute: (Atribut defaultValue: 1/100000 lock: false)
		at: 'Rel error'.
	self addAtribute: (Atribut defaultValue: nil lock: false)
		at: 'Max step'.
	self addAtribute: (Atribut defaultValue: nil lock: false)
		at: 'Min step'.
	self addAtribute: (Atribut defaultValue: 'RKE4' lock: false)
		at: 'Method'.! !

GraphicNode subclass: #StructuredNode
	instanceVariableNames: 'subStructure '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!StructuredNode methodsFor: 'accessing'!

label: aString 
	"Nastavi titulek uzlu a rovnez nazev jeho podstruktury."

	self subStructure == nil ifFalse: [self subStructure title = aString ifFalse: [self subStructure title: aString]].
	label _ aString!

subStructure
	"vraci ukazatel na podstrukturu obsazenou na nizsi hierarchicke urovni"

	^subStructure!

subStructure: aDocument
	"nastavi ukazatel na podstrukturu obsazenou na nizsi hierarchicke urovni"

	subStructure _ aDocument! !

!StructuredNode methodsFor: 'fileIn/Out'!

storeBinaryOn: stream

	super storeBinaryOn: stream.
	stream nextPut: self subStructure.! !

!StructuredNode methodsFor: 'compiling'!

compile
	"Kompilacni metoda uzlu. Provede kompilaci podstruktury."

	self subStructure == nil ifTrue: [^self confirm: 'Uzel s cislem ' , self number , ' nema definovanou podstrukturu'].
	^self subStructure compileNodes!

initCompilation
	"Inicializuje uzel pred kompilaci"

	super initCompilation.
	subStructure == nil ifFalse: [subStructure initCompilation]!

initConnections
	"Inicializuje uzel pred simulaci"

	subStructure == nil ifFalse: [subStructure initConnections]!

initSimulation
	"Inicializuje uzel pred simulaci"

	subStructure == nil ifFalse: [subStructure initSimulation]! !

!StructuredNode methodsFor: 'private'!

deepcopy
	| newObject |
	newObject _ self copy.
	newObject subStructure: self subStructure copy.
	^newObject!

getImage
	| form line pen |
	form _ super getImage.
	line _ Line new.
	pen _ Form extent: 1 @ 1.
	pen black.
	line form: pen.
	line beginPoint: form width * 0.9 @ 0.
	line endPoint: form width * 0.9 @ form height.
	line displayOn: form.
	^form! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

StructuredNode class
	instanceVariableNames: ''!


!StructuredNode class methodsFor: 'instance creation'!

readBinaryFrom: stream tag: tag
	"Vraci object precteny ze streamu"

	| newNode sd|
	sd _ ModelarDocument active.
	newNode _ super readBinaryFrom: stream tag: tag.
	newNode subStructure: stream next.
	newNode subStructure == nil ifFalse: [newNode subStructure superDocument: sd].
	ModelarDocument activate: sd.
	^newNode! !

GraphicNode subclass: #CommandNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!CommandNode methodsFor: 'simulation'!

behaviorIn: aSimProcess
	"Chovani procesu aSimProcess v uzlu."

	^self!

next
	"vraci uzel, kterym pokracuje pruchod grafem, ktery predstavuje simulacni 
	model"

	nOutputs = 0 ifTrue: [^nil].
	^outputs at: 1! !

!CommandNode methodsFor: 'ioLimits'!

minInputs
	^1!

minOuputs
	^0! !

GraphicNode subclass: #DataNode
	instanceVariableNames: 'data '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!DataNode methodsFor: 'accessing'!

data
	"Vraci datovou strukturu, kterou obsahuje"

	^data!

data: anObject 
	"Vlozi danny objekt jako datovou strukturu do uzlu"

	data _ anObject! !

!DataNode methodsFor: 'compiling'!

syntax
	^true! !

!DataNode methodsFor: 'simulation'!

initSimulation
	"Zaregistruje uzel v hlavnim uzlu simulace. To je nutne pro ziskani statistickych 
	udaju o simulaci, ktere jsou takto shromazdovany v hlavnim uzlu."


	self register!

register
	"Registruje uzel v hlavnim uzlu simulace. Jde hlavne o zpristupneni koncoveho 
	stavu uzlu po skonceni simulace - histogramy, statistiky zarizeni a skladu ..."

	| node |
	node _ ModelarDocument active topDocument nodeWithNumber: 0.
	node registerNode: self! !

!DataNode methodsFor: 'private'!

deepCopy
	data _ nil.
	^super deepCopy! !

DataNode subclass: #ContNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!ContNode methodsFor: 'compiling'!

initConnections
	| sum |
	nInputs > 1
		ifTrue: 
			[sum _ Sum new.
			inputs do: [:input | sum sum: input data].
			self data = sum].
	nInputs = 1 ifTrue: [self data = (inputs at: 1) data]!

initSimulation
	^self subclassResponsibility! !

ContNode subclass: #InputNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!InputNode methodsFor: 'ioLimits'!

maxInputs
	^0! !

Paragraph subclass: #MyParagraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!MyParagraph methodsFor: 'displaying'!

displayOn: aDisplayMedium at: aDisplayPoint clippingBox: clipRectangle rule: ruleInteger mask: aForm 
	

	super
		displayOn: aDisplayMedium
		at: aDisplayPoint + self offset
		clippingBox: clipRectangle
		rule: ruleInteger
		mask: aForm.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MyParagraph class
	instanceVariableNames: ''!


!MyParagraph class methodsFor: 'instance creation'!

string: aString 
	^self withText: aString asText! !

SimProcess subclass: #SimProcessTm
	instanceVariableNames: 'timers '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-componets'!


!SimProcessTm methodsFor: 'accessing'!

timeAt: aTimer 
	"vraci cas ulozeny v casovaci s cislem aTimer"

	^timers at: aTimer!

timeAt: aTimer put: time
	"Do casovace s cislem aTimer nastavi casovou informaci."

	timers at: aTimer put: time! !

!SimProcessTm methodsFor: 'initialize-release'!

initialize
	"inicializuje simulacni proces s casovaci"

	
	timers _ Array new: 10.
	MySimulation active == nil
		ifTrue: [timers at: 1 put: 0]
		ifFalse: [timers at: 1 put: MySimulation time]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimProcessTm class
	instanceVariableNames: ''!


!SimProcessTm class methodsFor: 'instance creation'!

new
	^super new initialize! !