StandardSystemController subclass: #SimInspectorController
	instanceVariableNames: ''
	classVariableNames: 'InspBlueButtonMenu InspBlueButtonMessages InspMenuCollapsed '
	poolDictionaries: ''
	category: 'Sim-Interface'!
SimInspectorController comment:
'Trida ''SimInspectorController'' definuje ridici objekt (controller) rozhrani simulacniho systemu. Trida dedi ze sve supertridy ''StandardSystemController'' vsechny atributy ridiciho objektu, vcetne akci zajistovanych obrazovym menu (supertrida definuje akce obrazoveho menu). Predefinovava nazvy polozek tohoto menu (ceske ekvivalenty) pro cely obraz rozhrani a pro obraz stahnuty do ikonky. Trida predefinovava metodu ''close'' pro uzavreni rozhrani - soucasne se zavrenim je treba ukoncit simulaci vsech modelu.

Promenne tridy:
  SimBlueButtonMenu <ActionMenu> - obrazove menu pro cely obraz rozhrani (nestazeny do ikonky)
  SimMenuCollapsed <ActionMenu> 	- obrazove menu pro obraz rozhrani stazeny do ikonky
  SimBlueButtonMessages <Array> 	- metody volane pri vyberu polozky z obrazoveho menu
 '!


!SimInspectorController methodsFor: 'initialization'!

initialize
	"Inicializuje konkretni ridici objekt - jeho obrazova menu a promenne 
	supertridy."

	super initialize.
	self initializeBlueButtonMenu! !

!SimInspectorController methodsFor: 'private'!

initializeBlueButtonMenu
	"Vola metodu supertridy pro nastaveni noveho obrazoveho menu, 
	ktere se bude aktivovat pri stisknuti obrazoveho tlacitka mysi."

	self blueButtonMenu: InspBlueButtonMenu blueButtonMessages: InspBlueButtonMessages! !

!SimInspectorController methodsFor: 'menu messages'!

blueButtonActivity
	"Rozliseni akci pri zmacknuti praveho (obrazoveho) tlacitka mysi pro 
	roztahnute okno a okno stahnute do ikony."

	| selector |
	view isCollapsed
		ifTrue: [(selector := InspMenuCollapsed startUp) ~= 0 ifTrue: [self menuMessageReceiver perform: (InspMenuCollapsed selectorAt: selector)]]
		ifFalse: [^super blueButtonActivity]!

close
	"Zavira obraz inspektoru simulacniho modelu."

	(model isMemberOf: (Smalltalk at: #SimModelInspector))
		ifTrue: [model object phase ~= #TERMINATION
				ifTrue: [(self confirm: 'Opravdu zavrit toto okno a ukoncit simulaci ?')
						ifTrue: 
							[super close.
							model object stop]]
				ifFalse: [super close]]
		ifFalse: [super close]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimInspectorController class
	instanceVariableNames: ''!


!SimInspectorController class methodsFor: 'class initialization'!

initialize
	"Inicializuje promenne tridy ridicich objektu inspektoru - obrazova 
	menu a akce s nimi spojene."

	InspMenuCollapsed := ActionMenu
				labels: 'Nove oznaceni\Okno pod\Posuv\Roztahnout ikonu\Zavrit' withCRs
				lines: #(1 4 )
				selectors: #(#newLabel #under #move #expand #close ).
	InspBlueButtonMessages := #(#newLabel #under #move #frame #collapse #close ).
	InspBlueButtonMenu := PopUpMenu labels: 'Nove oznaceni\Okno pod\Posuv\Roztahnuti\Ikona\Zavrit' withCRs lines: #(1 5 )! !

SimInspectorController initialize!


StandardSystemController subclass: #SimModelController
	instanceVariableNames: ''
	classVariableNames: 'SimBlueButtonMenu SimBlueButtonMessages SimMenuCollapsed '
	poolDictionaries: ''
	category: 'Sim-Interface'!
SimModelController comment:
'Trida ''SimModelController'' definuje ridici objekt inspektoru (controller) simulacniho modelu, a tim i inspektoru zarizeni, skladu, ... (subtrid tridy ''SimModelInspector''). Trida dedi ze sve supertridy ''StandardSystemController'' vsechny atributy ridiciho objektu, vcetne akci zajistovanych obrazovym menu (supertrida definuje akce obrazoveho menu). Predefinovava nazvy polozek tohoto menu (ceske ekvivalenty) pro cely obraz inspektoru a pro obraz stahnuty do ikonky. Trida predefinovava metodu ''close'' pro uzavreni obrazu inspektoru - soucasne se zavrenim je treba ukoncit simulaci prohlizeneho modelu.

Promenne tridy:
  InspBlueButtonMenu <ActionMenu> - obrazove menu pro cely obraz inspektoru (nestazeny do ikonky)
  InspMenuCollapsed <ActionMenu> 	- obrazove menu pro obraz inspektoru stazeny do ikonky
  InspBlueButtonMessages <Array> 	- metody volane pri vyberu polozky z obrazoveho menu
 
'!


!SimModelController methodsFor: 'initialization'!

initialize
	"Inicializuje konkretni ridici objekt - jeho obrazova menu a promenne 
	supertridy."

	super initialize.
	self initializeBlueButtonMenu! !

!SimModelController methodsFor: 'private'!

initializeBlueButtonMenu
	"Vola metodu supertridy pro nastaveni noveho obrazoveho menu, 
	ktere se bude aktivovat pri stisknuti obrazoveho tlacitka mysi."

	self blueButtonMenu: SimBlueButtonMenu blueButtonMessages: SimBlueButtonMessages! !

!SimModelController methodsFor: 'menu messages'!

blueButtonActivity
	"Rozliseni akci pri zmacknuti praveho (obrazoveho) tlacitka mysi pro 
	roztahnute okno a okno stahnute do ikony."

	| selector |
	view isCollapsed
		ifTrue: [(selector := SimMenuCollapsed startUp) ~= 0 ifTrue: [self menuMessageReceiver perform: (SimMenuCollapsed selectorAt: selector)]]
		ifFalse: [^super blueButtonActivity]!

close
	"Zavira obraz rozhrani simulacniho systemu."

	MySimulation active isNil ifFalse: [(self confirm: 'Opravdu zavrit toto okno a ukoncit vsechny simulace ?')
			ifTrue: 
				[MySimulation clear.
				super close]]
		ifTrue: [(self confirm: 'Opravdu zavrit simulacni okno ?')
				ifTrue: 
					[MySimulation clear.
					super close]]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimModelController class
	instanceVariableNames: ''!


!SimModelController class methodsFor: 'class initialization'!

initialize
	"Inicializuje promenne tridy ridicich objektu simulacniho rozhrani - 
	obrazova menu a akce s nimi spojene."

	SimMenuCollapsed := ActionMenu
				labels: 'Nove oznaceni\Okno pod\Posuv\Roztahnout ikonu\Zavrit' withCRs
				lines: #(1 4 )
				selectors: #(#newLabel #under #move #expand #close ).
	SimBlueButtonMessages := #(#newLabel #under #move #frame #collapse #close ).
	SimBlueButtonMenu := PopUpMenu labels: 'Nove oznaceni\Okno pod\Posuv\Zmena velikosti\Stahnout do ikony\Zavrit' withCRs lines: #(1 5 )! !

SimModelController initialize!


Inspector subclass: #SimModelInspector
	instanceVariableNames: 'fields simModel '
	classVariableNames: 'InspectorMenu InspTextMenu '
	poolDictionaries: ''
	category: 'Sim-Interface'!
SimModelInspector comment:
'Instance tridy ''SimModelInspector'' predstavuje model inspektoru simulacniho modelu. Ze sve supertridy ''Inspector'' dedi vsechny metody pro hierarchicke prohlizeni specifikovanych objektu. Predefinovava jeji operacni menu v obou subobrazech inspektoru (obraz pro seznam polozek a obraz pro prezentaci hodnoty zvolene polozky) - pridava polozku ''Ukonceni simulace'', ktera umozni zastavit simulaci prezentovaneho sim. modelu. Dale predefinovava seznam polozek urcenych k prohlizeni - polozky tohoto inspektoru tvori atributy modelu (vsechny objekty ve forme instancnich promennych) a atributy simulacniho experimentu. Zmenena je take metoda pro otevreni noveho inspektoru pro zvolenou polozku (inspect) - pro objekty typu zarizeni, sklad a fronta vola jejich vlastni metodu (inspect: simModel) pro otevreni specialniho inspektoru techto objektu.

Promenne tridy:
  InspectorMenu <ActionMenu> - operacni menu pro seznam polozek 
  InspTextMenu <ActionMenu> - operacni menu pro textovou prezentaci hodnoty zvolene polozky seznamu

Promenne instanci:
  fields <Array> - seznam polozek pro prohlizeni atributu simulacniho modelu a jeho experimentu
  simModel <MySimulation> - pdkaz na model prohlizeny inspektorem (pouzivany v subtridach teto tridy - QueueInspector, FacilityInspector a StoreInspector'!


!SimModelInspector methodsFor: 'text'!

textMenu
	"Definuje menu pro textovou cast inspektoru."

	InspTextMenu == nil ifTrue: [InspTextMenu := ActionMenu
					labels: 'Kopie textu\Vystrihnuti\Vlozeni\Inspektor objektu' withCRs
					lines: #(3 )
					selectors: #(#copySelection #cut #paste #inspectIt )].
	^InspTextMenu! !

!SimModelInspector methodsFor: 'field list'!

fieldList
	"Definuje seznam atributu simulacniho modelu urcenych k prohlizeni v 
	inspektoru."

	object == nil ifTrue: [^nil].
	fields := #('Tsim aktualni' 'Tsim pocatecni' 'Tsim koncovy' 'Faze' 'Entit celkem' 'Entit aktivnich' 'Kalendar' 'Priznak zastaveni' ) asOrderedCollection.
	fields addAllLast: object class instVarNames asOrderedCollection.
	^fields!

fieldMenu
	"Definuje menu pro praci s polozkami inspektoru."

	field == nil ifTrue: [^ActionMenu labels: 'ukonceni simulace' selectors: #(#cancelSimulation )].
	self fieldIndex isNil ifTrue: [^nil].
	InspectorMenu == nil ifTrue: [InspectorMenu := ActionMenu labels: 'inspect\ukonceni simulace' withCRs selectors: #(#inspectField #cancelSimulation )].
	^InspectorMenu!

fieldValue
	"Definuje hodnoty prezentovane inspektorem pro zvolenou polozku z 
	daneho seznamu."

	field isNil ifTrue: [^nil].
	field = 'self' ifTrue: [^object].
	field = 'Tsim aktualni' ifTrue: [^object time].
	field = 'Tsim pocatecni' ifTrue: [^object startTime].
	field = 'Tsim koncovy' ifTrue: [^object endTime].
	field = 'Faze' ifTrue: [^object phase].
	field = 'Kalendar' ifTrue: [^object calendar].
	field = 'Entit celkem' ifTrue: [^object entities].
	field = 'Entit aktivnich' ifTrue: [^object entitiesNow].
	field = 'Priznak zastaveni' ifTrue: [^object flag].
	^object instVarAt: self fieldIndex! !

!SimModelInspector methodsFor: 'menu messages'!

cancelSimulation
	"Ukonci simulaci prohlizeneho modelu."

	simModel stop!

inspectField
	"Otevre inspekcni okno pro zvolenou polozku - pro objekty ze trid 
	'Facility', 'Store' a 'Queue' vola jejich privatni metodu pro otevreni 
	specialniho inspektoru."

	| item |
	item := self fieldValue.
	(item isKindOf: (Smalltalk at: #Facility))
		| (item isKindOf: (Smalltalk at: #Store)) | (item isKindOf: (Smalltalk at: #Queue))
		ifTrue: [(object isKindOf: (Smalltalk at: #MySimulation))
				ifTrue: ["je to okno pro simulacni model"
					self fieldValue inspect: object]
				ifFalse: ["je to okno nejakeho objektu"
					self fieldValue inspect: simModel]]
		ifFalse: [self fieldValue inspect]! !

!SimModelInspector methodsFor: 'initialization'!

initialize: simmodel 
	"Inicializuje promennou inspektoru predstavujici odkaz na prohlizeny 
	simulacni model."

	simModel := simmodel! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimModelInspector class
	instanceVariableNames: ''!


!SimModelInspector class methodsFor: 'class initialization'!

flushMenus
	"Inicializuje menu inspektoru simulacniho modelu."

	InspectorMenu := InspTextMenu := nil! !

SimModelInspector subclass: #HistogramInspector
	instanceVariableNames: ''
	classVariableNames: 'HistInspectorMenu '
	poolDictionaries: ''
	category: 'Sim-Interface'!
HistogramInspector comment:
'Definuje model inspektoru pro prohlizeni atributu histogramu (instance tridy ''Histogram''). Zavadi nove operacni menu pro seznam polozek (doplnen o moznost vypisu histogramu na systemovy vystup). Predefinovava metody supertridy pro definici seznamu polozek (fieldList) a jejich prezentovanych hodnot (fieldValue).

Promenne tridy:

  HistInspectorMenu <ActionMenu> - operacni menu pro seznam polozek inspektoru'!


!HistogramInspector methodsFor: 'field list'!

fieldList
	"Definuje seznam atributu histogramu urcenych k prohlizeni v 
	inspektoru."

	object == nil ifTrue: [^nil].
	fields := #('Nazev' 'Dolni mez' 'Horni mez' 'Krok zaznamu' 'Pocet zaznamu' 'Max. hodnota' 'Min. hodnota' 'Prum. hodnota' ) asOrderedCollection.
	^fields!

fieldMenu
	"Definuje operacni menu pro praci s polozkami inspektoru."

	field == nil ifTrue: [^ActionMenu labels: 'ukonceni simulace\vypis histogramu' withCRs selectors: #(#cancelSimulation #print )].
	self fieldIndex isNil ifTrue: [^nil].
	HistInspectorMenu == nil ifTrue: [HistInspectorMenu := ActionMenu labels: 'inspect\vypis histogramu\ukonceni simulace' withCRs selectors: #(#inspectField #print #cancelSimulation )].
	^HistInspectorMenu!

fieldValue
	"Definuje hodnoty prezentovane inspektorem pro jednotlive polozky 
	jeho seznamu."

	field isNil ifTrue: [^nil].
	field = 'self' ifTrue: [^object].
	field = 'Nazev' ifTrue: [^object getName].
	field = 'Dolni mez' ifTrue: [^object lower].
	field = 'Horni mez' ifTrue: [^object upper].
	field = 'Krok zaznamu' ifTrue: [^object step].
	field = 'Pocet zaznamu' ifTrue: [^object stat num].
	field = 'Max. hodnota' ifTrue: [object stat num > 0
			ifTrue: [^object stat max]
			ifFalse: [^'nedefinovana']].
	field = 'Min. hodnota' ifTrue: [object stat num > 0
			ifTrue: [^object stat min]
			ifFalse: [^'nedefinovana']].
	field = 'Prum. hodnota' ifTrue: [object stat num > 0
			ifTrue: [^(object stat sumX / object stat num) asFloat]
			ifFalse: [^'nedefinovana']]! !

!HistogramInspector methodsFor: 'menu messages'!

print
	"Realizuje vypis histogramu do system. okna System Transcript."

	object output: Transcript.
	Transcript endEntry! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

HistogramInspector class
	instanceVariableNames: ''!


!HistogramInspector class methodsFor: 'class initialization'!

flushMenus
	"Inicializuje operacni menu inspektoru histogramu."

	HistInspectorMenu := nil! !

SimModelInspector subclass: #QueueInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sim-Interface'!
QueueInspector comment:
'Definuje model inspektoru pro prohlizeni atributu fronty (instance tridy ''Queue''). Operacni menu zustava stejne jako v jeji supertride ''SimModelInspector''. Predefinovava metody supertridy pro definici seznamu polozek (fieldList) a jejich prezentovanych hodnot (fieldValue).
'!


!QueueInspector methodsFor: 'field list'!

fieldList
	"Definuje seznam atributu fronty urcenych k prohlizeni v inspektoru."

	object == nil ifTrue: [^nil].
	fields := #('Jmeno' 'Vstoupilo entit' 'Vystoupilo' 'Max. delka' 'Prum. delka' 'Prum. doba cekani' ) asOrderedCollection.
	^fields!

fieldValue
	"Definuje hodnoty prezentovane inspektorem pro jednotlive polozky 
	jeho seznamu."

	field isNil ifTrue: [^nil].
	field = 'self' ifTrue: [^object].
	field = 'Jmeno' ifTrue: [^object getName].
	field = 'Vstoupilo entit' ifTrue: [^object statN num].
	field = 'Vystoupilo' ifTrue: [^object statDT num].
	field = 'Max. delka' ifTrue: [^object statN max].
	field = 'Prum. delka'
		ifTrue: 
			[simModel time > object statN tInit ifTrue: [^object statN sumXt + (object statN xLast * (simModel time - object statN tLast)) / (simModel time - object statN tInit)].
			^'nedefinovano'].
	field = 'Prum. doba cekani'
		ifTrue: 
			[object statDT num > 0 ifTrue: [^object statDT sumX / object statDT num].
			^'nedefinovano']! !

SimModelInspector subclass: #FacilityInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sim-Interface'!
FacilityInspector comment:
'Definuje model inspektoru pro prohlizeni atributu zarizeni (instance tridy ''Facility''). Operacni menu zustava stejne jako v jeji supertride ''SimModelInspector''. Predefinovava metody supertridy pro definici seznamu polozek (fieldList) a jejich prezentovanych hodnot (fieldValue).
'!


!FacilityInspector methodsFor: 'field list'!

fieldList
	"Definuje seznam atributu zarizeni urcenych k prohlizeni v inspektoru."

	object == nil ifTrue: [^nil].
	fields := #('Jmeno' 'Pocet pozadavku' 'Prum. vyuziti' 'Stav' 'Obsazeno entitou' 'Typ fronty' 'Vstupni fronta' ) asOrderedCollection.
	^fields!

fieldValue
	"Definuje hodnoty prezentovane inspektorem pro jednotlive polozky 
	jeho seznamu."

	field isNil ifTrue: [^nil].
	field = 'self' ifTrue: [^object].
	field = 'Jmeno' ifTrue: [^object getName].
	field = 'Pocet pozadavku' ifTrue: [^object tstat num].
	field = 'Prum. vyuziti'
		ifTrue: 
			[simModel time > object tstat tInit ifTrue: [^object tstat sumXt + (object tstat xLast * (simModel time - object tstat tLast)) / (simModel time - object tstat tInit)].
			^'nedefinovano'].
	field = 'Stav' ifTrue: [object busy
			ifTrue: [^'obsazeno']
			ifFalse: [^'neobsazeno']].
	field = 'Obsazeno entitou' ifTrue: [^object in].
	field = 'Vstupni fronta' ifTrue: [^object queueIn].
	field = 'Typ fronty' ifTrue: [object tQueue == #internal
			ifTrue: [^'interni fronta']
			ifFalse: [^'externi fronta']]! !

SimModelInspector subclass: #StoreInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sim-Interface'!
StoreInspector comment:
'Definuje model inspektoru pro prohlizeni atributu skladu (instance tridy Store). Operacni menu zustava stejne jako v jeji supertride ''SimModelInspector''. Predefinovava metody supertridy pro definici seznamu polozek (fieldList) a jejich prezentovanych hodnot (fieldValue).
'!


!StoreInspector methodsFor: 'field list'!

fieldList
	"Definuje seznam atributu skladu urcenych k prohlizeni v inspektoru."

	object == nil ifTrue: [^nil].
	fields := #('Jmeno' 'Kapacita' 'Obsazena kapacita' 'Max. obsazena kap.' 'Prum. obsazena kap.' 'Vstupni fronta' ) asOrderedCollection.
	^fields!

fieldValue
	"Definuje hodnoty prezentovane inspektorem pro jednotlive polozky jeho seznamu."

	field isNil ifTrue: [^nil].
	field = 'self' ifTrue: [^object].
	field = 'Jmeno' ifTrue: [^object getName].
	field = 'Kapacita' ifTrue: [^object capacity].
	field = 'Obsazena kapacita' ifTrue: [^object used].
	field = 'Max. obsazena kap.' ifTrue: [^object tstat max].
	field = 'Prum. obsazena kap.'
		ifTrue: 
			[simModel time > object tstat tInit ifTrue: [^object tstat sumXt + (object tstat xLast * (simModel time - object tstat tLast)) / (simModel time - object tstat tInit)].
			^'nedefinovano'].
	field = 'Vstupni fronta' ifTrue: [^object queueIn]! !

StandardSystemView subclass: #SimBrowserView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sim-Interface'!
SimBrowserView comment:
'Trida ''SimBrowserView'' definuje strukturu obrazu pro uzivatelske rozhrani simulacniho systemu. Dedi tridu ''StandardSystemView'' - navrzeny obraz ma tedy charakteristiky systemoveho obrazu. Obraz rozhrani je tvoren tremi subobrazy:
1. subobraz pro praci s tridami simulacnich modelu
2. subobraz pro praci s tridami simulacnich entit
3. subobraz pro prezentaci systemoveho vystupu (Transcript)
Implicitnim ridicim objektem tohoto obrazu je instance tridy ''SimModelController'', jejim modelem je instance tridy ''SimBrowser'', ktera definuje vsechny potrebne operace pro ovladani celeho rozhrani.'!


!SimBrowserView methodsFor: 'subview creation'!

addEntitySubView: area on: aBrowser readOnly: RO 
	"Definuje subobraz rozhrani simulacniho systemu pro praci s tridami 
	entit. Ten se sklada ze dvou subobrazu - subobrazu pro seznam trid 
	entit a subobrazu pro prezentaci komentare vybrane tridy 
	uvedeneho seznamu."

	| mid |
	mid := area left + area right * 0.5.
	self
		addSubView: (SelectionInListView
				on: aBrowser
				printItems: false
				oneItem: RO
				aspect: #entityClassName
				change: #entityClassName:
				list: #entityClassList
				menu: #entityClassMenu
				initialSelection: #entityClassName)
		in: (area copy right: mid)
		borderWidth: 1.
	self
		addSubView: (CodeView
				on: aBrowser
				aspect: #entityText
				change: #acceptTextE:from:
				menu: #entityTextMenu
				initialSelection: nil)
		in: (area copy left: mid)
		borderWidth: 1!

addModelSubView: area on: aBrowser readOnly: RO 
	"Definuje subobraz rozhrani simulacniho systemu pro praci s tridami 
	simulacnich modelu. Ten se sklada ze dvou subobrazu - subobrazu pro 
	seznam trid simulacnich modelu a subobrazu pro prezentaci 
	komentare vybrane tridy uvedeneho seznamu."

	| mid |
	mid _ area left + area right * 0.5.
	self
		addSubView: (SelectionInListView
				on: aBrowser
				printItems: false
				oneItem: RO
				aspect: #modelClassName
				change: #modelClassName:
				list: #modelClassList
				menu: #modelClassMenu
				initialSelection: #modelClassName)
		in: (area copy right: mid)
		borderWidth: 1.
	self
		addSubView: (CodeView
				on: aBrowser
				aspect: #modelText
				change: #acceptTextM:from:
				menu: #modelTextMenu
				initialSelection: nil)
		in: (area copy left: mid)
		borderWidth: 1!

addTextView: area on: aBrowser 
	"Definuje subobraz rozhrani simulacniho systemu pro praci se 
	systemovym vystupem (System Transcript), ktery se stava modelem 
	tohoto obrazu."

	| pomMenu pomMessages myView |
	pomMenu := PopUpMenu labels: 'Opakovat akci\Zrusit akci\Kopie textu\Vystrihnuti textu\Vlozeni textu\Spustit zvoleny blok\Spustit blok+vystup\Inspektor objektu\Ulozeni zmen\Zruseni zmen' withCRs lines: #(2 5 8 ).
	pomMessages := #(#again #undo #copySelection #cut #paste #doIt #printIt #inspectIt #accept #cancel ).
	Transcript clear.
	self
		addSubView: (myView := TextCollectorView new)
		in: area
		borderWidth: 1.
	myView model: (Smalltalk at: #Transcript).
	myView insideColor: Form white.
	myView borderWidth: 1.
	myView controller yellowButtonMenu: pomMenu yellowButtonMessages: pomMessages! !

!SimBrowserView methodsFor: 'controller access'!

defaultControllerClass
	"Vraci tridu definujici implicitni ridici objekt tohoto obrazu."

	^SimModelController! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimBrowserView class
	instanceVariableNames: ''!


!SimBrowserView class methodsFor: 'instance creation'!

open
	"Vytvori obraz pro rozhrani simulacniho systemu, ktery se sklada ze 
	tri subobrazu - obrazu pro praci s tridami simulacnich modelu, obrazu 
	pro praci s tridami entit a obrazu pro prezentaci systemoveho 
	vystupu (System Transcript)."

	| topView browser |
	SimBrowser flushMenus.
	browser := SimBrowser new.
	(topView := self
				model: browser
				label: ' DISKRETNI  SIMULATOR  v. 1.0 , autor:  Jaroslav JANOS  (C)1993'
				minimumSize: 500 @ 350)
		addModelSubView: (0 @ 0 extent: 0.5 @ 0.6)
		on: browser
		readOnly: false;
		addEntitySubView: (0.5 @ 0 extent: 0.5 @ 0.6)
		on: browser
		readOnly: false; addTextView: (0 @ 0.6 extent: 1.0 @ 0.4)
		on: browser.
	topView controller open! !

StandardSystemView subclass: #SimInspectorView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sim-Interface'!
SimInspectorView comment:
'Trida zavadi strukturu obrazu pro inspektor simulacniho modelu a inspektory simulacnich objektu (zarizeni, sklad, fronta a histogram). Obraz je tvoren dvema subobrazy - subobrazem pro seznam polozek inspektoru (s moznosti volby jedne polozky - instance tridy SelectionInListView) a subobrazem pro textovou prezentaci hodnoty zvolene polozky (instance tridy CodeView). Obraz inspektoru dedi tridu ''StandardSystemView'', ma tedy vsechny charakteristiky systemoveho obrazu.'!


!SimInspectorView methodsFor: 'controller access'!

defaultControllerClass
	"Vraci tridu definujici implicitni ridici objekt tohoto obrazu."

	^SimInspectorController! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimInspectorView class
	instanceVariableNames: ''!


!SimInspectorView class methodsFor: 'instance creation'!

open: anInspector label: lab 
	"Vytvori obraz inspektoru simulacniho modelu, ktery se sklada ze dvou 
	subobrazu - obrazu pro seznam polozek inspektoru a obrazu 
	prezentujiciho hodnotu zvolene polozky."

	| topView |
	topView _ self
				model: anInspector
				label: lab
				minimumSize: 180 @ 180.
	topView
		addSubView: (SelectionInListView
				on: anInspector
				printItems: anInspector printItems
				oneItem: false
				aspect: #field
				change: #field:
				list: #fieldList
				menu: #fieldMenu
				initialSelection: #field)
		in: (0 @ 0 extent: 0.3 @ 1)
		borderWidth: 1.
	topView
		addSubView: (CodeView
				on: anInspector
				aspect: #text
				change: #acceptText:from:
				menu: #textMenu
				initialSelection: nil)
		in: (0.3 @ 0 extent: 0.7 @ 1)
		borderWidth: 1.
	^topView controller open! !

Model subclass: #SimBrowser
	instanceVariableNames: 'modelClassName entityClassName text '
	classVariableNames: 'EntityMenu EntityTextMenu ModelMenu ModelTextMenu Organization TextMenu '
	poolDictionaries: ''
	category: 'Sim-Interface'!
SimBrowser comment:
'Instance tridy ''SimBrowser'' predstavuje model navrzeneho uzivatelskeho rozhrani simulacniho systemu. Definuje kategorie metod pro praci s tridami simulacnich modelu a tridami entit, a pro praci s textem v subobrazu pro systemovy vystup a v subobrazech pro prohlizeni komentaru trid.
Dale zavadi vsechny metody pro praci s operacnimi menu jednotlivych subobrazu, zajistujici i aktualizaci prislusnych obrazovych prezentaci vsech informaci (zakladem je zavedeni zavislosti mezi objekty tvoricimi rozhrani - napr. mezi zvolenou polozkou v seznamu trid a prislusnym subobrazem prezentujicim textovy komentar teto tridy).

Promenne tridy:
  EntityMenu <ActionMenu> 	- operacni menu pro seznam trid entit
  EntityTextMenu <ActionMenu> - operacni menu pro komentar zvolene tridy entity
  ModelMenu  <ActionMenu>  - operacni menu pro seznam trid simulacnich modelu
  ModelTextMenu <ActionMenu> - operacni menu pro komentar zvolene tridy sim. modelu
  Organization <SystemOrganizer> 
  TextMenu <ActionMenu> - operacni menu pro subobraz prezentujici systemovy vystup

Promenne instanci:
  modelClassName <String> - jmeno tridy predstavujici zvolenou polozku seznamu trid simulacnich modelu
  entityClassName <String> - jmeno tridy predstavujici zvolenou polozku seznamu trid entit
  text <WriteStream> 		- text systemoveho vystupu'!


!SimBrowser methodsFor: 'private-class functions'!

addNewClassE
	"Otevre okno pro editaci nove tridy z kategorie simulacni entity."

	self editCategoryE!

addNewClassM
	"Otevre okno pro editaci nove tridy z kategorie simulacni modely 
	(priklady)"

	self editCategoryM!

browseClassReferences: category 
	"Vraci seznam vsech odkazu na danou tridu."

	| pom |
	pom := category = $M
				ifTrue: [modelClassName]
				ifFalse: [entityClassName].
	Smalltalk browseAllCallsOn: (Smalltalk associationAt: pom)!

cancelSimulation
	"Zrusi vsechny spustene simulace."

	MySimulation clear!

editCategoryE
	"Otevre okno pro editaci trid z kategorie simulacni entity."

	self changed: #entityClassName.
	BrowserView openCategoryBrowserOn: ((Browser new) on: SystemOrganization; newCategoryList: #SimEntities)!

editCategoryM
	"Otevre okno pro editaci trid z kategorie simulacni modely (priklady)."

	BrowserView openCategoryBrowserOn: ((Browser new) on: SystemOrganization; newCategoryList: #SimExamples)!

editClassE
	"Otevre okno pro editaci tridy z kategorie simulacni entity."

	entityClassName isNil
		ifTrue: [^self]
		ifFalse: [self selEntityClass browse]!

editClassM
	"Otevre okno pro editaci tridy z kategorie simulacni modely (priklady)."

	modelClassName isNil
		ifTrue: [^self]
		ifFalse: [self selModelClass browse]!

fileInClass: category 
	"Nacte z externiho souboru tridu do Smalltalku, tj. prelozi jeji 
	textovou formu do formy binarni."

	| fileName |
	category = $M
		ifTrue: [fileName := Filename
						request: 'Zadejte jmeno souboru, z nehoz se bude cist'
						initially: 'model.st'
						shouldExist: nil]
		ifFalse: [fileName := Filename
						request: 'Zadejte jmeno souboru, z nehoz se bude cist'
						initially: 'entity.st'
						shouldExist: nil].
	fileName = '' | (fileName = ' ') ifTrue: [^self].
	(Filename named: fileName) fileIn.
	category = $M
		ifTrue: [self newModelClassList: modelClassName]
		ifFalse: [self newEntityClassList: entityClassName]!

fileInClassE
	"Nacte z externiho souboru tridu entit do Smalltalku, tj. prelozi jeji 
	textovou formu do formy binarni."

	self fileInClass: $E!

fileInClassM
	"Nacte z externiho souboru tridu simulacniho modelu do Smalltalku, tj. 
	prelozi jeji textovou formu do formy binarni."

	self fileInClass: $M!

fileOutClass: category 
	"Zapise zvolenou tridu do externiho souboru."

	| fileName fileStream |
	category = $M
		ifTrue: [fileName := Filename
						request: 'Zadejte jmeno souboru, do nehoz se bude zapisovat '
						initially: self modelClassName , '.st'
						shouldExist: nil]
		ifFalse: [fileName := Filename
						request: 'Zadejte jmeno souboru, do nehoz se bude zapisovat '
						initially: self entityClassName , '.st'
						shouldExist: nil].
	fileName = '' | (fileName = ' ') ifTrue: [^self].
	fileStream := (Filename named: fileName) writeStream.
	
	[fileStream timeStamp.
	category = $M
		ifTrue: [self selModelClass fileOutOn: fileStream]
		ifFalse: [self selEntityClass fileOutOn: fileStream]]
		valueNowOrOnUnwindDo: [fileStream close].
	category = $M
		ifTrue: [self selModelClass removeFromChanges]
		ifFalse: [self selEntityClass removeFromChanges]!

fileOutClassE
	"Zapise zvolenou tridu z kategorie simulacni entity do externiho 
	souboru."

	entityClassName isNil ifTrue: [^self].
	self fileOutClass: $E!

fileOutClassM
	"Zapise zvolenou tridu z kategorie simulacni priklady do externiho 
	souboru."

	modelClassName isNil ifTrue: [^self].
	self fileOutClass: $M!

initSimulation
	"Inicializuje simulacni system, tj. tridu 'MySimulation'."

	| pom |
	pom := MySimulation active.
	pom isNil ifFalse: [pom phase == #SIMULATION ifTrue: [(self confirm: 'Prave probiha simulace instance tridy `' , pom class name , '`' , '
 Opravdu chcete inicializovat simulacni sytem ?')
				ifTrue: [MySimulation initialize]]]
		ifTrue: [MySimulation initialize]!

removeClass: category 
	"Zrusi zvolenou tridu z kategorie entit nebo sim. modelu."

	| class |
	self changeRequest ifFalse: [^self].
	Dictionary keyNotFoundSignal
		handle: 
			[:ex | 
			category = $M
				ifTrue: [Transcript cr; show: 'Nelze zrusit tridu: `' , modelClassName , '` - uz neexistuje.']
				ifFalse: [Transcript cr; show: 'Nelze zrusit tridu: `' , entityClassName , '` - uz neexistuje.'].
			ex return]
		do: 
			["KeyNotFoundSignal je nastaven, byla-li trida zrusena z 
			jineho browseru"
			category = $M
				ifTrue: [class := self selModelClass]
				ifFalse: [class := self selEntityClass].
			(self confirm: 'Opravdu chcete zrusit tridu `' , class name , '` ?')
				ifTrue: 
					[class subclasses size > 0 ifTrue: [self notify: 'Trida `' , class name , '` ma podtridy.'].
					class removeFromSystem.
					category = $M
						ifTrue: [self newModelClassList: nil]
						ifFalse: [self newEntityClassList: nil]]]!

removeClassE
	"Zrusi tridu z kategorie simulacni entity."

	entityClassName isNil ifTrue: [^self].
	self removeClass: $E!

removeClassM
	"Zrusi tridu z kategorie simulacni priklady."

	modelClassName isNil ifTrue: [^self].
	self removeClass: $M!

renameClass: category 
	"Prejmenuje zvolenou tridu z kategorie entit nebo sim. modelu."

	| aString newName cleanString |
	self changeRequest ifFalse: [^self].
	category = $M
		ifTrue: [aString := self prompt: '   Zadejte nove jmeno tridy ' initially: self modelClassName]
		ifFalse: [aString := self prompt: '   Zadejte nove jmeno tridy ' initially: self entityClassName].
	aString isEmpty | (aString = ' ') ifTrue: [^self].
	cleanString := aString select: [:char | char isLetter or: [char isDigit]].
	cleanString = aString
		ifFalse: 
			[aString := cleanString.
			(self confirm: 'Nevhodne zadane jmeno tridy. Pouzit jmeno `' , aString , '` ?')
				ifFalse: [^self]].
	aString first isLowercase
		ifTrue: 
			[aString at: 1 put: (aString at: 1) asUppercase.
			(self confirm: 'Jmeno tridy musi zacinat velkym pismenem. Pouzit jmeno `' , aString , '` ?')
				ifFalse: [^self]].
	newName := aString asSymbol.
	category = $M
		ifTrue: [aString = self modelClassName
				ifFalse: 
					[self selModelClass rename: newName.	"trida z kategorie modelu - simulacni priklady"
					self newModelClassList: newName.
					Transcript cr; show: 'Hledam odkazy na tuto tridu ...'.
					self browseClassReferences: category]]
		ifFalse: [aString = self entityClassName
				ifFalse: 
					[self selEntityClass rename: newName.	"trida z kategorie simulacni entity"
					self newEntityClassList: newName.
					Transcript cr; show: 'Hledam odkazy na tuto tridu ...'.
					self browseClassReferences: category]]!

renameClassE
	"Prejmenuje zvolenou tridu z kategorie simulacni entity."

	entityClassName isNil ifTrue: [^self].
	self renameClass: $E!

renameClassM
	"Prejmenuje zvolenou tridu z kategorie simulacni priklady."

	modelClassName isNil ifTrue: [^self].
	self renameClass: $M!

simulate
	"Spusti simulaci instance zvolene tridy z kategorie sim. modelu."

	| simModel |
	modelClassName isNil
		ifFalse: 
			[Transcript show: 'Spusten simulacni model ze tridy `' , modelClassName , '` ...'; cr; endEntry.
			(simModel := self selModelClass new) run; output: Transcript.
			Transcript endEntry.
			simModel inspect]!

simulateSpool
	"Spusti simulaci instance zvolene tridy z kategorie sim. modelu na 
	pozadi."

	| process currModel |
	modelClassName isNil
		ifFalse: 
			[Transcript show: 'Na pozadi spusten simulacni model ze tridy `' , modelClassName , '` ...'; cr; endEntry.
			currModel := self selModelClass new.
			process := 
					[currModel run; output: Transcript.
					Transcript endEntry] newProcess.
			MySimulation addProcess: process with: currModel.
			currModel inspect]!

update
	"Aktualizuje udaje prezentovane obrazem simulacniho rozhrani."

	self changed: #entityClassName; changed: #modelClassName! !

!SimBrowser methodsFor: 'change management'!

logProtocolChange: aString 
	"Zaznamena zmenu do souboru zmen (prejmenovani tridy, zruseni ...)"

	Smalltalk logChange: aString! !

!SimBrowser methodsFor: 'text'!

acceptTextE: aText from: aController 
	"Ulozi zmeny textu komentare tridy entit provedene pri jeho editaci."

	self entityClassName == nil
		ifTrue: [aController flash]
		ifFalse: 
			[self selEntityClass comment: aText string.
			self changed: #entityText].
	^true!

acceptTextM: aText from: aController 
	"Ulozi zmeny textu komentare tridy simulacnich modelu provedene pri 
	jeho editaci."

	self modelClassName == nil
		ifTrue: [aController flash]
		ifFalse: 
			[self selModelClass comment: aText string.
			self changed: #modelText].
	^true!

entityText
	"Text u kategorie simulacni entity 
	-> pokud neni vybrana zadna trida, prazdny 
	-> je-li vybrana trida, pak zobrazi komentar k teto tride"

	| pom |
	entityClassName == nil
		ifTrue: [^Text new]
		ifFalse: 
			[pom := self selEntityClass comment asText.
			pom isEmpty ifFalse: [^pom].
			^'Tato trida nema komentar' asText]!

entityTextMenu
	"Vraci menu pro praci s textem v kategorii simulacni entity."

	^self textMenu: $E!

modelText
	"Text u kategorie simulacni modely 
	-> pokud neni vybrana zadna trida, prazdny 
	-> je-li vybrana trida, pak zobrazi komentar k teto tride"

	| pom |
	modelClassName == nil
		ifTrue: [^Text new]
		ifFalse: 
			[pom := self selModelClass comment asText.
			pom isEmpty ifFalse: [^pom].
			^'Tato trida nema komentar' asText]!

modelTextMenu
	"Vraci menu pro praci s textem v kategorii simulacni priklady."

	^self textMenu: $M!

prompt: promptString initially: initialString 
	"Okno pro zadavani retezce s dotazem `promptString` s implicitni 
	odpovedi `initialString`. 
	Uzivatel muze potvrdit implicitni volbu nebo zada novy retezec 
	predstavujici novou volbu."

	^FillInTheBlank request: promptString , '
a volbu potvrdte klavesou <Enter>' initialAnswer: initialString!

textMenu: category 
	"Definuje operacni menu pro praci s textem komentaru trid."

	category = $M
		ifTrue: 
			[ModelTextMenu == nil ifTrue: [ModelTextMenu := ActionMenu
							labels: 'Zapsat zmeny\Ignorovat zmeny' withCRs
							lines: #(1 )
							selectors: #(#accept #cancel )].
			^ModelTextMenu]
		ifFalse: 
			[EntityTextMenu == nil ifTrue: [EntityTextMenu := ActionMenu
							labels: 'Zapsat zmeny\Ignorovat zmeny' withCRs
							lines: #(1 )
							selectors: #(#accept #cancel )].
			^EntityTextMenu]! !

!SimBrowser methodsFor: 'class list'!

entityClassList
	"Vraci usporadany seznam trid pro kategorii simulacni entity."

	^Organization listAtCategoryNamed: #SimEntities!

entityClassMenu
	"Definuje operacni menu pro subobraz prezentujici seznam trid 
	kategorie simulacni entity."

	EntityMenu == nil ifTrue: [EntityMenu := ActionMenu
					labels: 'Zapsat do souboru\Precist ze souboru 
Editovat vybranou\Editovat vsechny\Vytvorit novou\Prejmenovat\Vymazat\Aktualizace okna' withCRs
					lines: #(2 5 7 )
					selectors: #(#fileOutClassE #fileInClassE #editClassE #editCategoryE #addNewClassE #renameClassE #removeClassE #update )].
	^EntityMenu!

entityClassName
	"Vraci jmeno prave vybrane tridy entit ze seznamu."

	^entityClassName!

entityClassName: selection 
	"Nastavi zvolenou tridu sim. entit v jejich seznamu; neexistuje-li, 
	vypise zpravu do system. okna."

	entityClassName := selection.
	Dictionary keyNotFoundSignal
		handle: 
			[:ex | 
			Transcript cr; show: 'Trida ' , entityClassName , ' uz neexistuje.'.
			ex return]
		do: ["Byla-li trida uz zrusena z jineho browseru"
			self changed: #entityText]!

modelClassList
	"Vraci usporadany seznam trid pro kategorii simulacni modelu."

	^Organization listAtCategoryNamed: #SimExamples!

modelClassMenu
	"Definuje operacni menu pro subobraz prezentujici seznam trid 
	kategorie simulacni modely."

	ModelMenu == nil ifTrue: [ModelMenu := ActionMenu
					labels: 'Zapsat do souboru\Precist ze souboru 
Editovat vybrany\Editovat vsechny\Vytvorit novy 
Simulovat na pozadi\Spustit simulaci\Zastavit vsechny simulace\Inicializace systemu
Prejmenovat\Vymazat\Aktualizace okna' withCRs
					lines: #(2 5 9 11 )
					selectors: #(#fileOutClassM #fileInClassM #editClassM #editCategoryM #addNewClassM #simulateSpool #simulate #cancelSimulation #initSimulation #renameClassM #removeClassM #update )].
	^ModelMenu!

modelClassName
	"Vraci jmeno prave vybrane tridy simulacnich modelu ze seznamu."

	^modelClassName!

modelClassName: selection 
	"Nastavi zvolenou tridu sim. modelu v jejich seznamu; neexistuje-li, 
	vypise zpravu do system. okna."

	modelClassName := selection.
	Dictionary keyNotFoundSignal
		handle: 
			[:ex | 
			Transcript cr; show: 'Trida ' , modelClassName , ' uz neexistuje.'.
			ex return]
		do: ["Byla-li trida uz zrusena z jineho browseru"
			self changed: #modelText]!

newEntityClassList: initialSelection 
	"Nastavi volbu prave vybrane tridy z kategorie simulacni entity."

	entityClassName := initialSelection.
	self changed: #entityClassName!

newModelClassList: initialSelection 
	"Nastavi volbu prave vybrane tridy z kategorie simulacni modely."

	modelClassName := initialSelection.
	self changed: #modelClassName!

selEntityClass
	"Vraci odkaz na prave vybranou tridu z kategorie sim. entit"

	^Smalltalk at: self entityClassName!

selModelClass
	"Vraci odkaz na prave vybranou tridu z kategorie sim. prikladu"

	^Smalltalk at: self modelClassName! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimBrowser class
	instanceVariableNames: ''!


!SimBrowser class methodsFor: 'class initialization'!

flushMenus
	"Inicializuje vsechna operacni menu a tridni promenne rozhrani sim. 
	systemu."

	ModelMenu := nil.
	EntityMenu := nil.
	ModelTextMenu := nil.
	EntityTextMenu := nil.
	TextMenu := nil.
	self initialize!

initialize
	"inicializace promennych tridy `SimBrowser`"

	Organization := SystemOrganization! !

SimBrowser initialize!
