BasicInputStep subclass: #Sinus
	instanceVariableNames: 'period phaseDisplace '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContSimInputs'!
Sinus comment:
' Trida ''Sinus'' je vstupni funkcni blok ''Harmonicky signal''.
Promenne instanci dedi po svem otci.

Promenne instanci:
phaseDisplace <Double>  - fazovy posuv signalu
period <Double>    - perioda impulsu'!


!Sinus methodsFor: 'accessing'!

getName
	"Vraci jmeno vstupniho bloku."

	^'Harmonicky signal '!

period
	"Vraci hodnotu periody."

	^period!

phaseDisplace
	"Vraci hodnotu fazoveho posunu."

	^phaseDisplace! !

!Sinus methodsFor: 'initialization'!

initialize: startTime step: newStep period: newPeriod phaseDisplace: newPhaseDisplacement 
	"Inicializuje vstupni blok 'Sinus'."

	super initialize: startTime step: newStep.
	newPeriod <= 0 ifTrue: [MySimulation error: 'Zadana frekvence u Sin je <= 0.'].
	period := newPeriod.
	newPhaseDisplacement < -180 ifTrue: [MySimulation error: 'Zadany fazovy posun u Sin musi byt v rozmezi <-180,180>.'].
	newPhaseDisplacement > 180 ifTrue: [MySimulation error: 'Zadany fazovy posun u Sin musi byt v rozmezi <-180,180>.'].
	phaseDisplace := newPhaseDisplacement! !

!Sinus methodsFor: 'task language'!

value
	"Vraci vystupni hodnotu vstupniho bloku."

	MySimulation time < timeStart
		ifTrue: [^0]
		ifFalse: [^(MySimulation time - timeStart \\ period / period * 360 + phaseDisplace) degreesToRadians sin * step]! !

!Sinus methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor harmonickeho signalu."

	SimInspectorView open: ((SinusInspector inspect: self)
			initialize: simModel)
		label: 'Harmonicky signal '! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Sinus class
	instanceVariableNames: ''!


!Sinus class methodsFor: 'instance creation'!

new: startTime step: newStep period: newPeriod phaseDisplace: newPhaseDisplacement 
	"Vytvari a inicializuje vstupni blok harmonickeho signalu."

	^super new
		initialize: startTime
		step: newStep
		period: newPeriod
		phaseDisplace: newPhaseDisplacement! !

BasicInputStep subclass: #Step
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContSimInputs'!
Step comment:
' Trida ''Step'' je vstupni funkcni blok ''Skok''.
Promenne instanci dedi po svem otci.'!


!Step methodsFor: 'accessing'!

getName
	"Vraci jmeno vstupniho bloku."

	^'Skok '! !

!Step methodsFor: 'task language'!

value
	"Vraci vystupni hodnotu vstupniho bloku."

	MySimulation time < timeStart
		ifTrue: [^0]
		ifFalse: [^step]! !

!Step methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor skoku."

	SimInspectorView open: ((StepInspector inspect: self)
			initialize: simModel)
		label: 'Skok '! !

BasicInputPulse subclass: #Pulse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContSimInputs'!
Pulse comment:
' Trida ''Pulse'' je vstupni funkcni blok ''Obdelnikovy impuls''.
Promenne instanci dedi po svem otci.'!


!Pulse methodsFor: 'accessing'!

getName
	"Vraci jmeno vstupniho bloku."

	^'Obdelnikovy impuls '! !

!Pulse methodsFor: 'task language'!

value
	"Vraci vystupni hodnotu vstupniho bloku."

	MySimulation time < timeStart
		ifTrue: [^0]
		ifFalse: 
			[MySimulation time <= (timeStart + duration) ifTrue: [^step].
			^0]! !

!Pulse methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor obdelnikoveho impulsu."

	SimInspectorView open: ((PulseInspector inspect: self)
			initialize: simModel)
		label: 'Obdelnikovy impuls '! !

Pulse subclass: #Pulses
	instanceVariableNames: 'period '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContSimInputs'!
Pulses comment:
' Trida ''Pulses'' je vstupni funkcni blok ''Obdelnikove impulsy''.
Promenne instanci dedi po svem otci.

Promenne instanci:
period <Double>   - perioda impulsu'!


!Pulses methodsFor: 'accessing'!

getName
	"Vraci jmeno vstupniho bloku."

	^'Obdelnikove impulsy '!

period
	"Vraci hodnotu periody."

	^period! !

!Pulses methodsFor: 'initialization'!

initialize: startTime step: newStep duration: newDuration period: newPeriod 
	"Inicializuje vstupni blok 'Pulses' na obdelnikove impulsy s periodou 
	'newPeriod'."

	super
		initialize: startTime
		step: newStep
		duration: newDuration.
	newPeriod <= newDuration ifTrue: [MySimulation error: 'Cas periody je <= delce trvani impulsu.'].
	period := newPeriod! !

!Pulses methodsFor: 'task language'!

value
	"Vraci vystupni hodnotu vstupniho bloku."

	| hPhase |
	MySimulation time < timeStart
		ifTrue: [^0]
		ifFalse: 
			[hPhase := MySimulation time - timeStart - (period * (MySimulation time - timeStart // period)).
			hPhase < duration ifTrue: [^step].
			^0]! !

!Pulses methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor obdelnikovych impulsu."

	SimInspectorView open: ((PulsesInspector inspect: self)
			initialize: simModel)
		label: 'Obdelnikove impulsy '! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Pulses class
	instanceVariableNames: ''!


!Pulses class methodsFor: 'instance creation'!

new: startTime 
	"Vytvari a inicializuje vstupni blok 'Pulses' na jednotkove obdelnikove 
	impulsy s periodou 2."

	^super new
		initialize: startTime
		step: 1
		duration: 1
		period: 2!

new: startTime step: newStep duration: newDuration period: newPeriod 
	"Vytvari a inicializuje vstupni blok 'Pulses' na obdelnikove impulsy s 
	periodou 'newPeriod'."

	^super new
		initialize: startTime
		step: newStep
		duration: newDuration
		period: newPeriod! !

BasicInput subclass: #Ramp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContSimInputs'!
Ramp comment:
' Trida ''Ramp'' je vstupni funkcni blok ''Skok rychlosti''.
Promenne instanci dedi po svem otci.'!


!Ramp methodsFor: 'accessing'!

getName
	"Vraci jmeno vstupniho bloku."

	^'Skok rychlosti '! !

!Ramp methodsFor: 'task language'!

value
	"Vraci vystupni hodnotu vstupniho bloku."

	MySimulation time < timeStart
		ifTrue: [^0]
		ifFalse: [^MySimulation time - timeStart]! !

!Ramp methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor skoku rychlosti."

	SimInspectorView open: ((RampInspector inspect: self)
			initialize: simModel)
		label: 'Skok rychlosti '! !

BasicInputPulse subclass: #Triangles
	instanceVariableNames: 'halfDuration '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContSimInputs'!
Triangles comment:
' Trida ''Triangles'' je vstupni funkcni blok ''Trojuhelnikovy impuls''.
Promenne instanci dedi po svem otci.

Promenne instanci:
halfDuration <Double>    - delka impulsu po vrchol'!


!Triangles methodsFor: 'accessing'!

getName
	"Vraci jmeno vstupniho bloku."

	^'Trojuhelnikovy impuls '!

halfDuration
	"Vraci trvani prvni casti trojuhelnikoveho impulsu."

	^halfDuration! !

!Triangles methodsFor: 'initialization'!

initialize: startTime step: newStep duration: newDuration halfDuration: newHalfDuration 
	"Inicializuje vstupni blok 'Triangles' na trojuhelnikovy impuls."

	super
		initialize: startTime
		step: newStep
		duration: newDuration.
	newHalfDuration < 0 ifTrue: [MySimulation error: 'Trvani prvni casti impulsu je < 0.'].
	newHalfDuration > duration ifTrue: [MySimulation error: 'Trvani prvni casti impulsu je > nez delka impulsu.'].
	halfDuration := newHalfDuration! !

!Triangles methodsFor: 'task language'!

value
	"Vraci vystupni hodnotu vstupniho bloku."

	MySimulation time < timeStart
		ifTrue: [^0]
		ifFalse: [MySimulation time <= (timeStart + halfDuration)
				ifTrue: [^MySimulation time - timeStart / halfDuration * step]
				ifFalse: 
					[MySimulation time <= (timeStart + duration) ifTrue: [^1 - (MySimulation time - (timeStart + halfDuration) / (duration - halfDuration)) * step].
					^0]]! !

!Triangles methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor trojuhelnikoveho impulsu."

	SimInspectorView open: ((TrianglesInspector inspect: self)
			initialize: simModel)
		label: 'Trojuhelnikovy impuls '! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Triangles class
	instanceVariableNames: ''!


!Triangles class methodsFor: 'instance creation'!

new: startTime 
	"Vytvari a inicializuje vstupni blok 'Triangles' na jednotkovy 
	trojuhelnikovy impuls."

	^super new
		initialize: startTime
		step: 1
		duration: 1
		halfDuration: 0.5!

new: startTime step: newStep duration: newDuration halfDuration: newHalfDuration 
	"Vytvari a inicializuje vstupni blok 'Triangles' na trojuhelnikovy 
	impuls."

	^super new
		initialize: startTime
		step: newStep
		duration: newDuration
		halfDuration: newHalfDuration! !

Triangles subclass: #Triangleses
	instanceVariableNames: 'period '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContSimInputs'!
Triangleses comment:
' Trida ''Triangleses'' je vstupni funkcni blok ''Trojuhelnikove impulsy''.
Promenne instanci dedi po svem otci.

Promenne instanci:
period <Double>    - perioda impulsu'!


!Triangleses methodsFor: 'accessing'!

getName
	"Vraci jmeno vstupniho bloku."

	^'Trojuhelnikove impulsy '!

period
	"Vraci hodnotu periody."

	^period! !

!Triangleses methodsFor: 'initialization'!

initialize: startTime step: newStep duration: newDuration halfDuration: newHalfDuration period: newPeriod 
	"Inicializuje vstupni blok 'Triangleses' na trojuhelnikove impulsy."

	super
		initialize: startTime
		step: newStep
		duration: newDuration
		halfDuration: newHalfDuration.
	newPeriod < newDuration ifTrue: [MySimulation error: 'Cas periody je < delce trvani impulsu.'].
	period := newPeriod! !

!Triangleses methodsFor: 'task language'!

value
	"Vraci vystupni hodnotu vstupniho bloku."

	| helpTime |
	MySimulation time < timeStart
		ifTrue: [^0]
		ifFalse: 
			[helpTime := MySimulation time - timeStart.
			helpTime := helpTime - (helpTime // period * period).
			helpTime <= halfDuration
				ifTrue: [^helpTime / halfDuration * step]
				ifFalse: 
					[helpTime <= duration ifTrue: [^duration - helpTime / (duration - halfDuration) * step].
					^0]]! !

!Triangleses methodsFor: 'user interface'!

inspect: simModel 
	"Inspektor trojuhelnikovych impulsu."

	SimInspectorView open: ((TrianglesesInspector inspect: self)
			initialize: simModel)
		label: 'Trojuhelnikove impulsy '! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Triangleses class
	instanceVariableNames: ''!


!Triangleses class methodsFor: 'instance creation'!

new: startTime 
	"Vytvari a inicializuje vstupni blok 'Triangleses' na jednotkove 
	trojuhelnikove impulsy s periodou 2."

	^super new
		initialize: startTime
		step: 1
		duration: 1
		halfDuration: 0.5
		period: 2!

new: startTime step: newStep duration: newDuration halfDuration: newHalfDuration period: newPeriod 
	"Vytvari a inicializuje vstupni blok 'Triangleses' na trojuhelnikove 
	impulsy."

	^super new
		initialize: startTime
		step: newStep
		duration: newDuration
		halfDuration: newHalfDuration
		period: newPeriod! !