CommandNode subclass: #WaitNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
WaitNode comment:
'Realizuje cekani entity.
Atribut ''tw-rozlozeni'' nastavuje typ pravdepodobnostniho rozlozeni casovych intervalu mezi generovanim entit. Nabyva techto hodnot:
	''E'' - Exponencialini rozlozeni
		Atribut ''tw-A'' je stredni hodnota tohoto rozlozeni
	''U'' - Uniformni rozlozeni
		Atributy ''tw-a'' a ''tw-b'' jsou horni a dolni mez tohoto rozlozeni
	''K'' - Konstanta (intervaly maji konstantni delku)
		Atribut ''tw'' je delka intervalu
'!


!WaitNode methodsFor: 'compiling'!

syntax
	"Pokud je uzel Wait nasledovan dalsim uzlem musi to byt povelovy uzel"

	nOutputs > 0 ifTrue: [((outputs at: 1)
			isKindOf: CommandNode)
			ifFalse: [^self confirm: 'Naslednik neni povelovy uzel']].
	^true! !

!WaitNode methodsFor: 'inspection'!

defaultInspectorClass
	^GeneratorInspector! !

!WaitNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	"Popisuje chovani procesu v cekacim uzlu"

	| distr |
	distr _ (self atributeAt: 'tw-rozlozeni') atributValue.
	distr = 'E' ifTrue: [^aSimProcess wait: (Exponential mean: (self atributeAt: 'tw-A') atributValue)].
	distr = 'U' ifTrue: [^aSimProcess wait: (Uniform from: (self atributeAt: 'tw-a') atributValue to: (self atributeAt: 'tw-b') atributValue)].
	distr = 'K' ifTrue: [^aSimProcess wait: (self atributeAt: 'tw') atributValue]!

next
	^outputs at: 1! !

!WaitNode methodsFor: 'initialize-release'!

initialize
	"inicializuje vytvoreny uzel"

	super initialize.
	self addAtribute: (Atribut defaultValue: 'E' lock: false)
		at: 'tw-rozlozeni'.
	self addAtribute: (Atribut defaultValue: 100 lock: false)
		at: 'tw-A'.
	label _ 'Wait'.
	^self! !

!WaitNode methodsFor: 'ioLimits'!

maxOutputs
	^1! !

DataNode subclass: #QueueNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
QueueNode comment:
'Pasivni prvek fronta. Lze pouzit pouze jako parametr FacilityNode a StoreNode nebo jejich potomku.'!


!QueueNode methodsFor: 'simulation'!

initSimulation
	"Inicializuje frontu pred spustenim simulace."

	super initSimulation! !

!QueueNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Queue'.
	^self! !

!QueueNode methodsFor: 'compiling'!

syntax
	| qu |
	qu _ Queue newQ: self label.
	self data: qu.
	^true! !

CommandNode subclass: #ReleaseNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
ReleaseNode comment:
'Uvolni zadane zarizeni (FacilityNode). Pokud zarizeni neni obsazeno aktualni entitou, vyvola se chyba'!


!ReleaseNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Realease'! !

!ReleaseNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	aSimProcess release: (outputs at: 1) data!

next
	"Vraci uzel, kteremu ma byt predano rizeni"

	((outputs at: nOutputs)
		isKindOf: CommandNode)
		ifTrue: [^outputs at: nOutputs].
	^nil! !

!ReleaseNode methodsFor: 'compiling'!

syntax
	"Zkontroluje syntaktickou spravnost pouziti uzlu Release v grafu"

	((outputs at: 1)
		isKindOf: FacilityNode)
		ifFalse: [^self confirm: 'Chybi uzel Facility'].
	nOutputs > 1 ifTrue: [((outputs at: 2)
			isKindOf: CommandNode)
			ifFalse: [^self confirm: 'Naslednik neni povelovy uzel']].
	^true! !

!ReleaseNode methodsFor: 'ioLimits'!

maxOutputs
	^2! !

CommandNode subclass: #StartTimerNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
StartTimerNode comment:
'Provede start zadaneho casovace (ulozi do nej aktualni hodnotu simulacniho casu).
Atribut ''Timer'' urcuje, ktery casovac ma byt spusten'!


!StartTimerNode methodsFor: 'initialize-release'!

initialize
	"Inicializuje uzel StartTimer, ktery odstartuje pozadovany meric casu v 
	procesu."

	super initialize.
	label _ 'Start timer'.
	self addAtribute: (Atribut defaultValue: 1 lock: false)
		at: 'Timer'! !

!StartTimerNode methodsFor: 'compiling'!

syntax
	"Naslednikem uzlu muze byt pouze uzel typu 'CommandNode'. Atribut udavajici 
	zvoleny casovac musi byt cislo v rozsahu 1 - 10."

	| tm |
	nOutputs > 0 ifTrue: [((outputs at: 1)
			isKindOf: CommandNode)
			ifFalse: [^self confirm: 'Nenasleduje povelovy uzel']].
	tm _ (self atributeAt: 'Timer') atributValue.
	(tm isKindOf: SmallInteger)
		ifTrue: [tm > 0 & tm < 11 ifTrue: [^true]].
	^false! !

!StartTimerNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	"Zapise aktualni simulacni cas do zvolenoho casovace v procesu aSimProcess."

	aSimProcess timeAt: (self atributeAt: 'Timer') atributValue put: MySimulation time!

next
	"Vraci uzel, kteremu bude predano rizeni."

	^outputs at: 1! !

DataNode subclass: #HistogramNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
HistogramNode comment:
'Histogram slouzi pro zaznam casovych intervalu. Zaznam provadi uzel WriteTimeNode.
Atributy ''Od'', ''Do'' urcuji interval zaznamenavanych hodnot
Atribut ''Krok'' urcuje sirku podintervalu, do kterych budou hodnoty rozdelovany'!


!HistogramNode methodsFor: 'initialize-release'!

initialize
	"Inicializuje uzel histogramu, nastavi jeho atributy na implicitni hodnoty."

	super initialize.
	self addAtribute: (Atribut defaultValue: 0 lock: false)
		at: 'Od'.
	self addAtribute: (Atribut defaultValue: 1000 lock: false)
		at: 'Do'.
	self addAtribute: (Atribut defaultValue: 1 lock: false)
		at: 'Krok'.
	label _ 'Histogram'! !

!HistogramNode methodsFor: 'ioLimits'!

maxOutputs
	^0!

minInputs
	^1! !

!HistogramNode methodsFor: 'simulation'!

initSimulation
	"Inicializuje histogram pred zahajenim simulace"

	| hst |
	hst _ Histogram
				new: self label
				from: (self atributeAt: 'Od') atributValue
				to: (self atributeAt: 'Do') atributValue
				by: (self atributeAt: 'Krok') atributValue.
	self data: hst.
	super initSimulation! !

!HistogramNode methodsFor: 'compiling'!

syntax
	^true! !

CommandNode subclass: #LeaveNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
LeaveNode comment:
'Uvolni kapacitu skladu (StoreNode), obsazenou entitou. Pokud ma atribut ''Kapacita'' hodnotu 0, uvolnuje se kapacita, kterou entita predtim skutecne obsadila, jinak se uvolni kapacita nastavena atributem (pozor na chybu!!!!!!)'!


!LeaveNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	"Uvolini ze skladu tolik polozek, kolik jich predtim obsadil, neni-li atributem Kapacita zadano jinak."

	(self atributeAt: 'Kapacita') atributValue = 0 ifFalse: [aSimProcess reqCap: (self atributeAt: 'Kapacita') atributValue].
	aSimProcess leave: (outputs at: 1) data!

next
	"Vraci uzel, kteremu bude predano rizeni"

	nOutputs > 1
		ifTrue: [^outputs at: 2]
		ifFalse: [^nil]! !

!LeaveNode methodsFor: 'compiling'!

syntax
	"Zkontroluje syntaktickou spravnost pouziti uzlu Leave v grafu"

	((outputs at: 1)
		isKindOf: StoreNode)
		ifFalse: [^self confirm: 'Chybi uzel Store'].
	nOutputs > 1 ifTrue: [((outputs at: 2)
			isKindOf: CommandNode)
			ifFalse: [^self confirm: 'Naslednik neni povelovy uzel']].
	^true! !

!LeaveNode methodsFor: 'initialize-release'!

initialize
	"Inicializuje uzel Leave."

	super initialize.
	self addAtribute: (Atribut defaultValue: 0 lock: false)
		at: 'Kapacita'.
	label _ 'Leave'! !

DataNode subclass: #FacilityNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
FacilityNode comment:
'Zarizeni s vylucnym pristupem. Jako parametr muze mit QueueNode predstavujici externi frontu, jinak se pouzije fronta interni.'!


!FacilityNode methodsFor: 'simulation'!

initSimulation
	"Inicializuje zarizeni pred spustenim simulace."

	| fac |
	fac _ Facility new: self label.
	nOutputs > 0 ifTrue: [fac setQueue: (outputs at: 1) data].
	self data: fac.
	super initSimulation! !

!FacilityNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Facility'.
	^self! !

!FacilityNode methodsFor: 'ioLimits'!

maxOutputs
	^1! !

!FacilityNode methodsFor: 'compiling'!

syntax
	nOutputs = 0 ifTrue: [^true].
	((outputs at: 1)
		isKindOf: QueueNode)
		ifFalse: [^self confirm: 'Naslednikem neni Queue'].
	^true! !

CommandNode subclass: #SeizeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
SeizeNode comment:
'Obsadi zarizeni s vylucnym pristupem - facility. Jako soused uzlu je pozadovan uzel FacilityNode nebo jeho potomek.
Atribut ''Priorita'' nastavuje prioritu entity pri obsazovani zarizeni.'!


!SeizeNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Seize'.
	self addAtribute: (Atribut defaultValue: 0 lock: false) at: 'Priorita'! !

!SeizeNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	aSimProcess seize: (outputs at: 1) data priority: (self atributeAt: 'Priorita') atributValue!

next
	"Vraci uzel, kteremu ma byt predano rizeni"

	nOutputs > 1
		ifTrue: [^outputs at: 2]
		ifFalse: [^nil]! !

!SeizeNode methodsFor: 'compiling'!

syntax
	"Zkontroluje syntaktickou spravnost pouziti uzlu Seize v grafu"

	((outputs at: 1)
		isKindOf: FacilityNode)
		ifFalse: [^self confirm: 'Chybi uzel Facility'].
	nOutputs > 1 ifTrue: [((outputs at: 2)
			isKindOf: CommandNode)
			ifFalse: [^self confirm: 'Naslednik neni povelovy uzel']].
	^true! !

!SeizeNode methodsFor: 'ioLimits'!

maxOutputs
	^2! !

SeizeNode subclass: #SeizeAnyNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
SeizeAnyNode comment:
'Obsadi nahodne jedno ze zarizeni (FacilityNode)
Atribut ''Priorita'' nastavuje prioritu entity pri obsazovani zarizeni.'!


!SeizeAnyNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	| fac nFac |
	((outputs at: nOutputs)
		isKindOf: FacilityNode)
		ifTrue: [nFac _ nOutputs]
		ifFalse: [nFac _ nOutputs - 1].
	fac _ outputs at: (Uniform from:1 to: nFac) rounded.
	aSimProcess seize: fac data priority: (self atributeAt: 'Priorita') atributValue!

next
	| last |
	last _ outputs at: nOutputs.
	(last isKindOf: CommandNode)
		ifTrue: [^last]
		ifFalse: [^nil]! !

!SeizeAnyNode methodsFor: 'compiling'!

syntax
	| node |
	node _ outputs detect: [:x | (x isKindOf: FacilityNode) not]
				ifNone: [^true].
	nOutputs = 1 ifTrue: [^self confirm: 'Chybi uzel Facility'].
	node == (outputs at: nOutputs)
		ifTrue: [(node isKindOf: CommandNode)
				ifFalse: [^self confirm: 'Naslednik neni povelovy uzel']]
		ifFalse: [^self confirm: 'Vice nez jeden naslednik'].
	^true! !

!SeizeAnyNode methodsFor: 'ioLimits'!

maxOutputs
	^10! !

!SeizeAnyNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Seize Any'.! !

SeizeAnyNode subclass: #SeizeProbNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
SeizeProbNode comment:
'Obsadi nektere ze zarizeni (sousedni uzly) podle zadane pravdepodobnosti.
Atributy ''p1'' - ''p9'' urcuji pravdepodobnost obsazeni i-teho zarizeni. Je-li pocet zrizeni mensi nez 9, musi byt nepouzite vystupy nastaveny na pravdepodobnost 0. Zarizeni jsou serazena podle cisel uzlu.
Atribut ''Priorita'' nastavuje prioritu entity pri obsazovani zarizeni.'!


!SeizeProbNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	| fac nFac prob k sum |
	((outputs at: nOutputs)
		isKindOf: FacilityNode)
		ifTrue: [nFac _ nOutputs]
		ifFalse: [nFac _ nOutputs - 1].
	prob _ Uniform from: 0 to: 1.
	sum _ 0.
	k _ 0.
	
	[k _ k + 1.
	sum _ sum + (self atributeAt: 'p' , k printString) atributValue.
	sum < prob & (k <= nFac)]
		whileTrue: [].
	k > nFac ifTrue: [k _ 1].
	fac _ outputs at: k.
	aSimProcess seize: fac data priority: (self atributeAt: 'Priorita') atributValue! !

!SeizeProbNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Seize Prob'.
	1 to: self maxOutputs - 1 do: [:i | self addAtribute: (Atribut defaultValue: 0 lock: false)
			at: 'p' , i printString].
	(self atributeAt: 'p1')
		atributValue: 1! !

SeizeAnyNode subclass: #SeizeFreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
SeizeFreeNode comment:
'Obsadi zarizeni (FacilityNode), ktere je volne. Neni-li volne zadne zarizeni, vybere nahodne jedno a zaradi se do jeho fronty.
Atribut ''Priorita'' nastavuje prioritu entity pri obsazovani zarizeni.'!


!SeizeFreeNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	| fac nFac i flag |
	((outputs at: nOutputs)
		isKindOf: FacilityNode)
		ifTrue: [nFac _ nOutputs]
		ifFalse: [nFac _ nOutputs - 1].
	i _ 0.
	flag _ false.
	
	[i _ i + 1.
	i <= nFac & flag not]
		whileTrue: 
			[fac _ outputs at: i.
			flag _ fac data busy not].
	flag ifTrue: [aSimProcess seize: fac data priority: (self atributeAt: 'Priorita') atributValue] ifFalse: [
	fac _ outputs at: (Uniform from: 1 to: nFac) rounded.
	aSimProcess seize: fac data priority: (self atributeAt: 'Priorita') atributValue]! !

!SeizeFreeNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Seize Free'.! !

SeizeAnyNode subclass: #ReleaseSeizedNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
ReleaseSeizedNode comment:
'Uvolni zarizeni obsazene entitou. Pokud entita neobsadila zadne zarizeni, nic se neprovede'!


!ReleaseSeizedNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Relese Seized'.
	self removeAtribute: 'Priorita'! !

!ReleaseSeizedNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	| fac nFac i flag |
	((outputs at: nOutputs)
		isKindOf: FacilityNode)
		ifTrue: [nFac _ nOutputs]
		ifFalse: [nFac _ nOutputs - 1].
	i _ 0.
	flag _ false.
	
	[i _ i + 1.
	(i <= nFac) & (flag not)]
		whileTrue: 
			[fac _ (outputs at: i) data.
			flag _ fac in == aSimProcess].
	flag ifTrue: [aSimProcess release: fac]! !

DataNode subclass: #StoreNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
StoreNode comment:
'Pasivni prvek se sdilenym pristupem - sklad.
Atribut ''Kapacita'' udava celkovou kapacitu skladu
Atribut ''Obsazeno'' udava pocatecni obsazeni'!


!StoreNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	self addAtribute: (Atribut defaultValue: 100 lock: false)
		at: 'Kapacita'.
	self addAtribute: (Atribut defaultValue: 0 lock: false)
		at: 'Obsazeno'.
	label _ 'Store'! !

!StoreNode methodsFor: 'compiling'!

syntax
	nOutputs = 0 ifTrue: [^true].
	((outputs at: 1)
		isKindOf: QueueNode)
		ifFalse: [^self confirm: 'Naslednikem neni Queue'].
	^true! !

!StoreNode methodsFor: 'ioLimits'!

maxOutputs
	^1! !

!StoreNode methodsFor: 'simulation'!

initSimulation
	"inicializuje sklad pred zahajenim simulace"

	| str |
	str _ Store new: self label capacity: (self atributeAt: 'Kapacita') atributValue.
	nOutputs > 0 ifTrue: [str setQueue: (outputs at: 1) data].
	(self atributeAt: 'Obsazeno') atributValue > 0 ifTrue: [str enter: (SimProcess new) capacity: (self atributeAt: 'Obsazeno') atributValue].
	self data: str.
	super initSimulation! !

GraphicNode subclass: #GeneratorNode
	instanceVariableNames: 'simproces '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
GeneratorNode comment:
'Generator slouzi ke generovani entit.
Atribut ''generuj od'' udava cas, od ktereho ma byt generator spusten.
Atribut ''generuj do'' udava cas zastaveni generatoru.
Atribut ''tw-rozlozeni'' nastavuje typ pravdepodobnostniho rozlozeni casovych intervalu mezi generovanim entit. Nabyva techto hodnot:
	''E'' - Exponencialini rozlozeni
		Atribut ''tw-A'' je stredni hodnota tohoto rozlozeni
	''U'' - Uniformni rozlozeni
		Atributy ''tw-a'' a ''tw-b'' jsou horni a dolni mez tohoto rozlozeni
	''K'' - Konstanta (intervaly maji konstantni delku)
		Atribut ''tw'' je delka intervalu
'!


!GeneratorNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	"chovani generatoru"

	[MySimulation time <= (self atributeAt: 'generuj do') atributValue]
		whileTrue: 
			[self generate.
			self wait: aSimProcess]!

generate
	"vygeneruje novou entitu pusti je do systemu"

	| node simprocess |
	node _ self next.
	simprocess _ SimProcessTm new.
	simprocess
		initialize: 
			[[node == nil]
				whileFalse: 
					[node behaviorIn: simprocess.
					node _ node next].
			simprocess terminate].
	simprocess activate!

initSimulation
	"Inicializuje generator entit. Naplanuje jeho spusteni v case zacatku simulace."

	| simProcess  |
	simProcess _ SimProcess new.
	simProcess initialize: [self behaviorIn: simProcess].
	simProcess activate: (self atributeAt: 'generuj od') atributValue!

next
	"Vraci souseda, kteremu bude predano rizeni v dalsim kroku."

	^outputs at: 1!

wait: aSimProcess 
	"Cekani mezi generovanim entit"

	| distr |
	distr _ (self atributeAt: 'tw-rozlozeni') atributValue.
	distr = 'E' ifTrue: [aSimProcess wait: (Exponential mean: (self atributeAt: 'tw-A') atributValue)].
	distr = 'U' ifTrue: [aSimProcess wait: (Uniform from: (self atributeAt: 'tw-a') atributValue to: (self atributeAt: 'tw-b') atributValue)].
	distr = 'K' ifTrue: [aSimProcess wait: (self atributeAt: 'tw') atributValue]! !

!GeneratorNode methodsFor: 'compiling'!

syntax
	"Zkontroluje zda uzel generatoru je nasledovan nejakym povelovym uzlem"

	nOutputs = 0 ifTrue: [^true].
	((outputs at: 1)
		isKindOf: CommandNode)
		ifFalse: [^self confirm: 'Chybi povelovy uzel'].
	^true! !

!GeneratorNode methodsFor: 'ioLimits'!

maxInputs
	^1!

maxOutputs
	^1!

minInputs
	^1!

minOutputs
	^0! !

!GeneratorNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Generator'.
	self addAtribute: (Atribut defaultValue: 'E' lock: false)
		at: 'tw-rozlozeni'.
	self addAtribute: (Atribut defaultValue: 100 lock: false)
		at: 'tw-A'.
	self addAtribute: (Atribut defaultValue: 0 lock: false)
		at: 'generuj od'.
	self addAtribute: (Atribut defaultValue: 1000 lock: false)
		at: 'generuj do'.
	^self! !

!GeneratorNode methodsFor: 'inspection'!

defaultInspectorClass
	^GeneratorInspector! !

CommandNode subclass: #WriteTimeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
WriteTimeNode comment:
'Zapisuje hodnotu casovace do histogramu.
Atribut ''Timer'' urcuje, ktery casovac se ma pouzit, muze nabyvat hodoty 1 - 10.
Casovac 1 je implicitni, startuje se pri vytvoreni entity. Jine casovace musi byt odstartovany pomoci uzlu StartTimerNode'!


!WriteTimeNode methodsFor: 'compiling'!

syntax
	"Zkontroluje syntaktickou spravnost pouziti uzlu WriteTime v grafu"

	| tm |
	((outputs at: 1)
		isKindOf: HistogramNode)
		ifFalse: [^self confirm: 'Chybi uzel Histogram'].
	nOutputs > 1 ifTrue: [((outputs at: 2)
			isKindOf: CommandNode)
			ifFalse: [^self confirm: 'Naslednik neni povelovy uzel']].
	tm _ (self atributeAt: 'Timer') atributValue.
	(tm isKindOf: SmallInteger)
		ifTrue: [tm > 0 & tm < 11 ifTrue: [^true]].
	^false! !

!WriteTimeNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	"Zapise cas z potricneho casovace do histogramu"

	| t |
	t _ aSimProcess timeAt: (self atributeAt: 'Timer') atributValue.
	t == nil ifTrue: [self error: 'Neinicializovany casovac'].
	t > MySimulation time ifTrue: [self error: 'Chybne nastaveni casovace'].
	t _ MySimulation time - t.
	(outputs at: 1) data store: t!

next
	"Vraci uzel, ktery bude simulace pokracovat."

	nOutputs > 1
		ifTrue: [^outputs at: 2]
		ifFalse: [^nil]! !

!WriteTimeNode methodsFor: 'initialize-release'!

initialize
	"Inicializuje uzel pro zapis casu do histogramu. O tom, ktery casovac ze 
	simulacniho procesu se pouzije rozhoduje atribut 'Timer'."

	super initialize.
	label _ 'WriteTime'.
	self addAtribute: (Atribut defaultValue: 1 lock: false)
		at: 'Timer'! !

CommandNode subclass: #BranchAnyNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
BranchAnyNode comment:
'Vybere nahodne jeden z vystupu a preda tam rizeni'!


!BranchAnyNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	"Neprovadi zadnou akci, pozadovanou fukci zajistuje metoda next"

	^self!

next
	nOutputs = 0 ifTrue: [^nil].
	^outputs at: (Uniform from: 1 to: nOutputs) rounded! !

!BranchAnyNode methodsFor: 'compiling'!

syntax
	nOutputs = 0 ifTrue: [^true].
	((outputs at: 1)
		isKindOf: CommandNode)
		ifFalse: [^self confirm: 'Chybi povelovy uzel'].
	^true! !

!BranchAnyNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Branch Any'.! !

BranchAnyNode subclass: #BranchProbNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
BranchProbNode comment:
'Vybere jeden z vystupu podle zadane pravdepodobnosti
Atributy ''p1'' - ''p9'' udavaji pravdepodobnosti vyberu i-teho vystupu. Je-li pocet zrizeni mensi nez 9, musi byt nepouzite vystupy nastaveny na pravdepodobnost 0. Zarizeni jsou serazena podle cisel uzlu.'!


!BranchProbNode methodsFor: 'simulation'!

next
	| prob sum k |
	nOutputs = 0 ifTrue: [^nil].
	prob _ Uniform from: 0 to: 1.
	sum _ 0.
	k _ 0.
	
	[k _ k + 1.
	sum _ sum + (self atributeAt: 'p' , k printString) atributValue.
	sum < prob & (k <= nOutputs)]
		whileTrue: [].
	k > nOutputs ifTrue: [k _ 1].
	^outputs at: k! !

!BranchProbNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Branch Prob'.
	1 to: self maxOutputs - 1 do: [:i | self addAtribute: (Atribut defaultValue: 0 lock: false)
			at: 'p' , i printString].
	(self atributeAt: 'p1')
		atributValue: 1! !

CommandNode subclass: #EnterNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!
EnterNode comment:
'Prikaz Enter slouzi k obsazovani skladu.
Atribut ''Kapacita'' udava, jak velka kapacita skladu bude entiou obsazena.'!


!EnterNode methodsFor: 'initialize-release'!

initialize
	"Inicializuje uzel Enter. Nastavi atribut 'Kapacita' udavajici obsazovanou 
	kapacitu skladu."

	super initialize.
	self addAtribute: (Atribut defaultValue: 10 lock: false)
		at: 'Kapacita'.
	label _ 'Enter'! !

!EnterNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	"Obsadi tolik polozek ve skladu kolik udava atribut 'Kapacita'."

	aSimProcess enter: (outputs at: 1) data capacity: (self atributeAt: 'Kapacita') atributValue!

next
	"Vraci uzel, kteremu bude predano rizeni"

	nOutputs > 1
		ifTrue: [^outputs at: 2]
		ifFalse: [^nil]! !

!EnterNode methodsFor: 'compiling'!

syntax
	"Zkontroluje syntaktickou spravnost pouziti uzlu Enter v grafu"

	((outputs at: 1)
		isKindOf: StoreNode)
		ifFalse: [^self confirm: 'Chybi uzel Store'].
	nOutputs > 1 ifTrue: [((outputs at: 2)
			isKindOf: CommandNode)
			ifFalse: [^self confirm: 'Naslednik neni povelovy uzel']].
	^true! !

EnterNode subclass: #EnterAnyNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Nodes-Diskret'!


!EnterAnyNode methodsFor: 'simulation'!

behaviorIn: aSimProcess 
	| str nStr |
	((outputs at: nOutputs)
		isKindOf: StoreNode)
		ifTrue: [nStr _ nOutputs]
		ifFalse: [nStr _ nOutputs - 1].
	str _ outputs at: (Uniform from: 1 to: nStr) rounded.
	aSimProcess enter: str data capacity: (self atributeAt: 'Kapacita') atributValue!

next
	| last |
	last _ outputs at: nOutputs.
	(last isKindOf: CommandNode)
		ifTrue: [^last]
		ifFalse: [^nil]! !

!EnterAnyNode methodsFor: 'compiling'!

syntax
	| node |
	node _ outputs detect: [:x | (x isKindOf: StoreNode) not]
				ifNone: [^true].
	nOutputs = 1 ifTrue: [^self confirm: 'Chybi uzel Store'].
	node == (outputs at: nOutputs)
		ifTrue: [(node isKindOf: CommandNode)
				ifFalse: [^self confirm: 'Naslednik neni povelovy uzel']]
		ifFalse: [^self confirm: 'Vice nez jeden naslednik'].
	^true! !

!EnterAnyNode methodsFor: 'ioLimits'!

maxOutputs
	^10! !

!EnterAnyNode methodsFor: 'initialize-release'!

initialize
	super initialize.
	label _ 'Enter Any'.! !