SimProcess subclass: #ObecnyGenerator
	instanceVariableNames: 'gName '
	classVariableNames: 'DefaultBehavior '
	poolDictionaries: ''
	category: 'SimEntities'!
ObecnyGenerator comment:
'Trida definujici generator entit s obecnym chovanim.
Chovani generatoru charakterizuji dva bloky :
- blok popisujici cekani generatoru mezi dvema reaktivacemi 
- blok popisujici chovani generatoru pri jeho aktivaci 

Promenna tridy:
  DefaultBehavior <BlockClosure> - blok predstavujici parametrizovatelne chovani jakehokoliv generatoru entit.

Promenna instanci:
  gName <String> - jmeno generatoru'!


!ObecnyGenerator methodsFor: 'accessing'!

getName
	"Vraci jmeno generatoru."

	^gName!

setName: newName 
	"Nastavi nove jmeno generatoru."

	gName := newName! !

!ObecnyGenerator methodsFor: 'initialization'!

initialize: newName wait: waitBlock behavior: behavIn 
	"Inicializuje chovani entity typu generator. Vyuziva pritom obecne 
	chovani generatoru definovane blokem s parametry, na nejz odkazuje 
	promenna tridy."

	super initialize: [DefaultBehavior
			value: self
			value: waitBlock
			value: behavIn].
	gName := newName! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ObecnyGenerator class
	instanceVariableNames: ''!
ObecnyGenerator class comment:
'Select the browser switch "instance" to see the comment  '!


!ObecnyGenerator class methodsFor: 'class initialization'!

initialize
	"Inicializuje promennou tridy definujici formou bloku s parametry 
	obecne chovani generatoru entit. Parametry bloku: 
	rec - konkretni generator 
	wBlock - blok popisujici cekani generatoru mezi reaktivaci 
	behIn - blok popisujici chovani generatoru pri aktivaci"

	DefaultBehavior := [:rec :wBlock :behIn | [true]
				whileTrue: 
					[behIn value.
					rec wait: wBlock value]]! !

!ObecnyGenerator class methodsFor: 'instance creation'!

new: gname wait: waitBlock behavior: behavIn 
	"Vytvari novou entitu s chovanim generatoru entit. Chovani 
	generatoru charakterizuji dva bloky: 
	wBlock - popisuje cekani generatoru mezi dvema reaktivacemi 
	behIn - blok popisujici chovani generatoru pri jeho aktivaci."

	self initialize.
	^super new
		initialize: gname
		wait: waitBlock
		behavior: behavIn! !

ObecnyGenerator initialize!


SimProcess subclass: #Zarizeni
	instanceVariableNames: 'prichod odchod '
	classVariableNames: 'DefaultBehavior '
	poolDictionaries: ''
	category: 'SimEntities'!
Zarizeni comment:
'Popisuje chovani entity pri jeji interakci s objektem typu zarizeni. 
Entita obsazuje zarizeni, je obsluhovana po stanovenou dobu a pote zarizeni uvolni.
Doba stravena entitou v systemu je zapsana do histogramu.

Promenne tridy:
  DefaultBehavior <BlockClosure> - blok definujici parametrizovatelne chovani entity pri jeji interakci se zarizenim
 
Promenne instanci:
  odchod <Float> - cas vstupu entity do simulacniho systemu
  prichod <Float> - cas ukonceni cinnosti entity v systemu'!


!Zarizeni methodsFor: 'initialization'!

initialize: fac wait: sTime tabulate: tab 
	"Inicializuje chovani entity, ktera je obsluhovana zarizenim. Vyuziva 
	pritom obecne chovani definovane v tridni promenne formou bloku s 
	parametry. Doba stravena entitou v systemu je zaznamenana do 
	histogramu."

	DefaultBehavior isNil ifTrue: [MySimulation active error: 'Trida `Zarizeni` jeste nebyla inicializovana.'].
	super
		initialize: 
			[prichod := MySimulation time.
			DefaultBehavior
				value: self
				value: fac
				value: sTime.
			odchod := MySimulation time.
			tab store: odchod - prichod]! !

!Zarizeni methodsFor: 'accessing'!

odchod
	"Vraci cas ukonceni obsluhy entity."

	^odchod!

prichod
	"Vraci cas vstupu entity do sim. systemu."

	^prichod! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Zarizeni class
	instanceVariableNames: ''!


!Zarizeni class methodsFor: 'class initialization'!

initialize
	"Inicializuje promennou tridy definujici formou bloku s parametry 
	obecne chovani entity v systemu s jednim zarizenim."

	DefaultBehavior := 
			[:rec :fac :sTime | 
			rec seize: fac.
			rec wait: sTime.
			rec release: fac]! !

!Zarizeni class methodsFor: 'instance creation'!

enter: fac service: sTime tabulate: tab 
	"Vytvari novou entitu s chovanim definujicim jeji interakci se 
	zarizenim 'fac' - entita obsazuje zarizeni 'fac' , je obsluhovana 
	po dobu 'sTime' a doba stravena entitou v systemu je zapsana 
	do histogramu 'tab'."

	^super new
		initialize: fac
		wait: sTime
		tabulate: tab! !

Zarizeni initialize!


Zarizeni subclass: #Zarizeni2Prefer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimEntities'!
Zarizeni2Prefer comment:
'Trida definuje chovani entity v systemu dvou zarizeni, ktere nemaji
spolecnou frontu. Entita obsadi volne zarizeni, v pripade, ze jsou obe
zarizeni obsazena, vstoupi do kratsi fronty k jednomu z nich.'!


!Zarizeni2Prefer methodsFor: 'initialization'!

initialize: fac1 or: fac2 service: sTime tabulate: tab 
	"Inicializuje atributy a chovani entity v systemu dvou zarizeni. Entita 
	obsadi zarizeni volne nebo zarizeni s kratsi vstupni frontou. Zaroven 
	se vyuziva obecneho chovani entity pri obsazovani zarizeni, ktere je 
	definovano v supertride a dostupne pomoci promenne tridy. Doba 
	stravena entitou v systemu je zaznamenana do histogramu."

	DefaultBehavior isNil ifTrue: [MySimulation active error: 'Trida `Zarizeni` jeste nebyla inicializovana'].
	super
		initialize: 
			[prichod := MySimulation time.
			fac1 busy not | (fac1 busy & fac2 busy & (fac1 queueIn size < fac2 queueIn size))
				ifTrue: [DefaultBehavior
						value: self
						value: fac1
						value: sTime]
				ifFalse: [DefaultBehavior
						value: self
						value: fac2
						value: sTime].
			odchod := MySimulation time.
			tab store: odchod - prichod]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Zarizeni2Prefer class
	instanceVariableNames: ''!


!Zarizeni2Prefer class methodsFor: 'instance creation'!

enter: fac1 or: fac2 service: sTime tabulate: tab 
	"Vytvari novou entitu s chovanim definujicim jeji interakci se 
	zarizenim 'fac1' nebo 'fac2` - entita obsazuje prvni volne zarizeni 
	nebo zarizeni s kratsi vstupni frontou. Doba stravena entitou v 
	systemu je zapsana do histogramu 'tab'"

	^super new
		initialize: fac1
		or: fac2
		service: sTime
		tabulate: tab! !

Zarizeni subclass: #Zarizeni2Spol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimEntities'!
Zarizeni2Spol comment:
'Trida definuje chovani entity v systemu dvou zarizeni se spolecnou frontou.
Entita pozadujici obsluhu obsadi volne zarizeni. Jsou-li obe zarizeni obsazene,
zaradi se do fronty. Po ukonceni obsluhy je touto entitou aktivovana prvni
entita cekajici na obsluhu ve fronte. '!


!Zarizeni2Spol methodsFor: 'initialization'!

initialize: aBlock1 seize: fac1 or: fac2 or: queue wait: sTime behAfter: aBlock2 tabulate: tab 
	"Inicializuje atributy a chovani entity v systemu 2 zarizeni se 
	spolecnou frontou. Entita bude obsazovat prvni volne zarizeni nebo 
	frontu. Zaroven se vyuziva obecneho chovani entity pri obsazovani 
	zarizeni, ktere je definovano v supertride a dostupne pomoci 
	promenne tridy. Argumenty 'aBlock1' a 'aBlock2' definuji chovani 
	entity pred a po jeji interakci se zarizenim. Doba stravena entitou v 
	systemu je zaznamenana do histogramu."

	DefaultBehavior isNil ifTrue: [MySimulation active error: 'Trida `Zarizeni` jeste nebyla inicializovana.'].
	super
		initialize: 
			[prichod := MySimulation time.
			aBlock1 value.
			fac1 busy & fac2 busy ifTrue: [self waitQ: queue].
			fac1 busy ifFalse: [DefaultBehavior
					value: self
					value: fac1
					value: sTime]
				ifTrue: [DefaultBehavior
						value: self
						value: fac2
						value: sTime].
			queue isEmpty ifFalse: [queue removeFirst activate].
			aBlock2 value.
			odchod := MySimulation time.
			tab store: odchod - prichod]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Zarizeni2Spol class
	instanceVariableNames: ''!


!Zarizeni2Spol class methodsFor: 'instance creation'!

enter: fac1 or: fac2 or: queue service: sTime tabulate: tab 
	"Vytvari novou entitu s chovanim definujicim obsazovani jednoho 
	ze dvou zarizeni 'fac1', 'fac2' se spolecnou frontou 'queue' - entita 
	obsadi volne zarizeni nebo ceka ve fronte, odkud ji z cela uvolni 
	obslouzena entita. Doba stravena entitou v systemu je zapsana do 
	histogramu 'tab'."

	^super new
		initialize: []
		seize: fac1
		or: fac2
		or: queue
		wait: sTime
		behAfter: []
		tabulate: tab! !

Zarizeni2Spol subclass: #ProgrUloha
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimEntities'!
ProgrUloha comment:
'Trida ''ProgrUloha'' definuje entity s chovanim uloh pro zpracovani
pocitacovym systemem. Uloha nejdrive obsazuje pamet, potom 
jeden ze dvou procesoru, ktery ji zpracovava. Delka zpracovani 
pozadavku se zaznamenava do tabulky. Trida vyuziva chovani
definovane ve sve supertride ''Zarizeni2Par'', ktera popisuje chovani
entity v systemu 2 zarizeni se spolecnou frontou.'!


!ProgrUloha methodsFor: 'initialization'!

initialize: pamet capacity: cap seize: proc1 or: proc2 or: queue service: sTime tabulate: tab 
	"Inicializuje atributy a chovani entity reprezentujici programovy 
	pozadavek v systemu se spolecnou pameti a dvema procesory se 
	spolecnou frontou. Pri definici chovani entity se vyuziva inicializacni 
	metoda supertridy, kterou lze parametrizovat pomoci bloku."

	super
		initialize: [self enter: pamet capacity: cap]
		seize: proc1
		or: proc2
		or: queue
		wait: sTime
		behAfter: [self leave: pamet]
		tabulate: tab! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ProgrUloha class
	instanceVariableNames: ''!


!ProgrUloha class methodsFor: 'instance creation'!

enter: pamet capacity: cap seize: proc1 or: proc2 or: queue service: sTime tabulate: tab 
	"Vytvari novou entitu s chovanim definujicim programovou ulohu 
	zpracovavanou procesorem. Uloha nejdrive obsadi pamet, potom 
	jeden ze dvou procesoru, ktery ji zpracovava. Delka zpracovani 
	pozadavku se zaznamenava do tabulky."

	^super new
		initialize: pamet
		capacity: cap
		seize: proc1
		or: proc2
		or: queue
		service: sTime
		tabulate: tab! !