MySimulation subclass: #BesselEx
	instanceVariableNames: 'int1 int2 name '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimExamples'!
BesselEx comment:
'Reprezentuje Besselovu diferencialni rovnici.'!


!BesselEx methodsFor: 'initialization'!

initialize: newName 
	"deklarace objektu simulacniho modelu"

	| graf cas |
	name := newName.
	graf := Graph new: 'Bessel1' by: 0.02.
	int1 := Integrator new: 'Integrator1' initVal: 0.4999.
	int2 := Integrator new: 'Integrator2' initVal: 0.01.
	cas := Tim new.
	
	"inicializacni faze simulacniho experimentu"
	self init: 0.02 to: 10.
	self setStep: 0.001 to: 0.01.

	int1 = (Sum new as: Inv new = (Mul new as: Div1 new = cas mul: int1) sum: (Mul new as: int2 mul: (Sum new as: -1 sum: Div1 new = (Sqr new = cas)))).
	int2 = int1.
	graf as: int1 o: int2! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BesselEx class
	instanceVariableNames: ''!


!BesselEx class methodsFor: 'instance creation'!

new
	"Vytvari novy simulacni model."

	^super new activate initialize: 'Bessel'! !

MySimulation subclass: #TestGener
	instanceVariableNames: 'tabulka mName predchozi '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimExamples'!
TestGener comment:
'Simulacni model predstavujici test generatoru exponencialniho rozlozeni. Model obsahuje jedinou entitu - generator, ktery se opakovane aktivuje a intervaly mezi reaktivacemi maji exponencialni rozlozeni. Delka techto intervalu je zapisovana do tabulky.

Promenne instanci:
  predchozi <Float> - cas predchozi aktivace generatoru
  tabulka <Histogram> - tabulka pro zaznam intervalu mezi dvema reaktivacemi generatoru
  mName <String> - jmeno modelu'!


!TestGener methodsFor: 'initialization'!

initialize: newName 
	"Inicializace simulacniho modelu - testovani gen. nah. cisel."

	mName := newName.
	predchozi := 0.
	tabulka := Histogram
				new: 'Test generatoru nah. cisel'
				from: 1
				to: 100
				by: 1.0.
	BasicRandom initialize.
	ObecnyGenerator initialize.
	self init: 0 to: 9999.
	(ObecnyGenerator
		new: ''
		wait: [Exponential mean: 10.0]
		behavior: 
			[tabulka store: self time - predchozi.
			predchozi := self time]) activate! !

!TestGener methodsFor: 'printing'!

output: aStream 
	"Vystup vysledku simulace - histogram predstavujici intervaly mezi 
	reaktivacemi generatoru entit zobrazi priblizny prubeh fce hustoty 
	pravdepodobnosti u exponenc. rozlozeni."

	tabulka output: aStream! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TestGener class
	instanceVariableNames: ''!


!TestGener class methodsFor: 'instance creation'!

new
	"Vytvoreni noveho modelu a inicializace jeho parametru."

	^super new activate initialize: 'Test generatoru'! !

MySimulation subclass: #LorenzEx
	instanceVariableNames: 'int1 int2 int3 name '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimExamples'!
LorenzEx comment:
'Reprezentuje Lorenzovy rovnice.'!


!LorenzEx methodsFor: 'initialization'!

initialize: newName 
	"deklarace objektu simulacniho modelu"

	| graf |
	name := newName.
	graf := Graph new: 'Lorenz' by: 0.02.
	int1 := Integrator new: 'Integrator1' initVal: -1.
	int2 := Integrator new: 'Integrator2'.
	int3 := Integrator new: 'Integrator3' initVal: 15.

	"inicializacni faze simulacniho experimentu"
	self init: 0 to: 100.
	self setAccuracy: 1 / 1000.

	int1 = (Mul new as: 10 mul: (Sum new as: int2 sum: Inv new = int1)).
	int2 = (Sum new
			as: (Mul new as: 28 mul: int1)
			sum: Inv new = int2
			sum: Inv new = (Mul new as: int1 mul: int3)).
	int3 = (Sum new as: (Mul new as: int1 mul: int2)
			sum: Inv new = (Mul new as: 2.66667 mul: int3)).
	graf
		as: int1
		o: int2
		o: int3! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LorenzEx class
	instanceVariableNames: ''!


!LorenzEx class methodsFor: 'instance creation'!

new
	"Vytvari novy simulacni model."

	^super new activate initialize: 'Lorenz'! !

MySimulation subclass: #ProgramSystem2
	instanceVariableNames: 'pamet proc1 proc2 spolFronta tabulka mName '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimExamples'!
ProgramSystem2 comment:
'Trida simulacnich modelu pocitacoveho systemu se dvema procesory a spolecnou pameti. Prichazejici pozadavky na zpracovani nejdrive obsazuji odpovidajici velikost pameti a pote zadaji o obsluhu jednim ze dvou procesoru. Po ukonceni zpracovani uvolni pozadavek i procesor i pamet. Na rozdil od tridy ''ProgramSystem'' jsou modely teto tridy nezavisle na tridach entit, protoze vsechny entity obsazene v modelu jsou vytvareny primo jako instance tridy ''SimProcess''. Z toho vyplyva i slozitejsi inicializacni metoda, ktera musi definovat chovani vsech entit v systemu.

Promenne instanci:
  pamet <Store> - spolecna pamet 
  proc1, proc2 <Facility> - procesory
  tabulka <Histogram> - tabulka pro zaznam dob zpracovani jednotlivych programovych uloh
  mName <String> - jmeno modelu'!


!ProgramSystem2 methodsFor: 'initialization'!

initialize: newName 
    "Na rozdil od tridy 'ProgramSystem' jsou modely teto tridy
     nezavisle na tridach entit, protoze vsechny entity obsazene
     v modelu jsou vytvareny primo jako instance tridy 'SimProcess'.
     Z toho vyplyva i slozitejsi inicializacni metoda, ktera musi
     definovat chovani vsech entit v systemu."

    | behav generbeh |

"deklarace objektu simulacniho modelu"
 	mName := newName.
	proc1 := Facility new: '1.Procesor'.
	proc2 := Facility new: '2.Procesor'.
	spolFronta := Queue newQ: 'Spolecna fronta uloh'.
     proc1 setQueue: spolFronta.
     proc2 setQueue: spolFronta.
	pamet := Store new: 'Pamet' capacity: 3000.	"spolecna pamet s kapacitou v MByte"
	tabulka := Histogram
				new: 'Doby zpracovani pozadavku programovym systemem'
				from: 1
				to: 500
				by: 50.	
 
"deklarace chovani entity reprezentujici ulohu urcenou ke zpracovani"
     behav := [ | rec prichod |           "odkaz na entitu a vstup entity do systemu"
                    rec := MySimulation active calendar current.
                    prichod := MySimulation time.
                    rec enter: pamet capacity: (Uniform from: 64 to: 1000) truncated. 
                    proc1 busy 
                        ifFalse: [rec seize: proc1]
                        ifTrue: [proc2 busy 
                                    ifFalse: [rec seize: proc2]
                                    ifTrue: [rec waitQ: spolFronta]
                                 ].
                    rec wait: (Exponential mean: 10.0).
                    proc1 in == rec 
                        ifTrue: [rec release: proc1]
                        ifFalse: [rec release: proc2].
                    rec leave: pamet.
                    tabulka store: MySimulation time - prichod           
                 ].

"deklarace chovani generatoru uloh"
       generbeh := [ | rec |
                        rec := MySimulation active calendar current.
                       [true]
                     whileTrue: 
                        [ (SimProcess new: [behav value]) activate.
                           rec wait: (Exponential mean: 5.0). 
                        ]
                   ].

"inicializacni faze simulacniho experimentu"
	self init: 0 to: 5000.       "simulacni cas bude prochazet intervalem <0,1000)"
"aktivace entit"
     (SimProcess new: generbeh) activate.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ProgramSystem2 class
	instanceVariableNames: ''!


!ProgramSystem2 class methodsFor: 'instance creation'!

new
	"Vytvari novy simulacni model pocitacoveho systemu se dvema 
	procesory a spolecnou pameti. Prichazejici pozadavky na zpracovani 
	nejdrive obsazuji odpovidajici velikost pameti a pote zadaji o obsluhu 
	jednim ze dvou procesoru. Po ukonceni zpracovani uvolni pozadavek i 
	procesor i pamet."

	^super new activate initialize: ''!

new: mName 
	"Vytvari novy simulacni model s nazvem mName. Cinnost stejna jako u 
	metody 'new'."

	^super new activate initialize: mName! !

MySimulation subclass: #ProgramSystem
	instanceVariableNames: 'pamet proc1 proc2 spolFronta tabulka mName '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimExamples'!
ProgramSystem comment:
'Trida simulacnich modelu pocitacoveho systemu se dvema procesory a spolecnou pameti. Prichazejici pozadavky na zpracovani nejdrive obsazuji odpovidajici velikost pameti a pote zadaji o obsluhu jednim ze dvou procesoru. Po ukonceni zpracovani uvolni pozadavek i procesor i pamet.

Promenne instanci:
  pamet <Store> - spolecna pamet 
  proc1, proc2 <Facility> - procesory
  tabulka <Histogram> - tabulka pro zaznam dob zpracovani jednotlivych programovych uloh
  mName <String> - jmeno modelu'!


!ProgramSystem methodsFor: 'initialization'!

initialize: newName 
    "Inicializuje atributy konkretniho simulacniho modelu. Vytvari
    a inicializuje objekty obsazene v simulovanem systemu, 
    tj. pamet (sklad), procesory (zarizeni), spolecnou frontu, 
    histogram atd. . Realizuje inicializacni fazi simulace a 
    vytvari generator uloh, ktery bude generovat entity
    ze tridy 'ProgrUloha'."

"deklarace objektu simulacniho modelu"
 	mName := newName.
	proc1 := Facility new: '1.Procesor'.
	proc2 := Facility new: '2.Procesor'.
	spolFronta := Queue newQ: 'Spolecna fronta uloh'.
	pamet := Store new: 'Pamet' capacity: 3000.	"spolecna pamet s kapacitou v kByte"
	tabulka := Histogram
				new: 'Doby zpracovani pozadavku programovym systemem'
				from: 1
				to: 500
				by: 50.	
 
"inicializacni faze simulacniho experimentu"
	self init: 0 to: 5000.       "simulacni cas bude prochazet intervalem <0,1000)"
	(ObecnyGenerator         "vytvoreni entity generator se zadanym chovanim"
		new: ''                  "a jeho aktivace"
		wait: [Exponential mean: 5.0]
		behavior: [ "chovani je urceno vytvorenim a aktivaci noveho pozadavku"
	 		(ProgrUloha
				enter: pamet
				capacity: (Uniform from: 64 to: 1000) truncated
				seize: proc1
				or: proc2
				or: spolFronta
				service: (Exponential mean: 10.0)
				tabulate: tabulka) activate]
      ) activate! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ProgramSystem class
	instanceVariableNames: ''!


!ProgramSystem class methodsFor: 'instance creation'!

new
	"Vytvari novy simulacni model pocitacoveho systemu se dvema 
	procesory a spolecnou pameti. Prichazejici pozadavky na zpracovani 
	nejdrive obsazuji odpovidajici velikost pameti a pote zadaji o obsluhu 
	jednim ze dvou procesoru. Po ukonceni zpracovani uvolni pozadavek i 
	procesor i pamet."

	^super new activate initialize: ''!

new: mName 
	"Vytvari novy simulacni model s nazvem mName. Cinnost stejna jako u 
	metody 'new'."

	^super new activate initialize: mName! !